# coding: utf-8
# This script allows the loading, plotting and fitting of a few differnt data types.
#     It has been written by Shae Padrick (SBP) to allow handling of some common datatypes.

# To run this script you'll need to call it from the command line as:
#     python scriptname setupfile ( > outputfile )

# You will need numpy, scipy and matplotlib installed for the script to work.  As of 2/1/2011 the most current version
# of python supported by all three of those packages is python2.6.  Thus, python 2.6 will need to be installed and the 
# call to python above should be set up to achieve that.  On my macbook, I have several versions of python in several 
# locations.  Thus I have my path in X11 set up to reference the correct python framework realtively early in the path
# and I call python2.6 instead of python.

# Changelog
# 2/1/2011 SBP I've got it working at a basic level.  The overall command and plotting structure is taking shape.
#              Currently I can read in data points from a setup file, fit them to direct binding or 
#              competition binding isotherms.  The fits can be overlayed on the data, and multiple
#              datasets and fits can be overlayed.
# 2/3/2011 SBP Added the ability to process fluorescence anisotropy data in the script from individual
#              files, as are collected on the PTI fluorimeter.  
#              Added the ability import arbitrary preprocesed (x, y) data.
#              Added a timecourse import section. 
# 2/26/2011 SBP  Added a comment function.  Any line in the setupfile beginning with "#" is a comment.
#              also, forgot to note this, but sometime around 2/14/2001 I added the ability to plot data with symbols
#              only appearing every n-th data point.
# 2/28/2011 SBP  Completed adding the basic forms of the AAAanalyze functions to v7.  All of these were brought in with
#              the ability to send info to the command show_processing.  This was happening by the functions writing to 
#              a global dictionary varible (pass_to_visproc) which resulted in difficult to read code.  I'm rewriting 
#              the AAAanalyze functions to be more straightforward input->func->output in behavior.  In the past these 
#              have been called with a few parameters, but several other parameters were passed as global variables.
#              Again, that is bad form.  So this will rewrite them to take all their info from formal parameter lists
#              and to write all their info out to formal results.  
# 3/20/2011 SBP  I'll note that there have been a few other additions between now and 2/28/2011.  
#              These include the addition of the meta_data command and some output_format flags in the AAAanalyze commands.  
#              mostly I've been just using the commands.  The change between v0.74 and v0.75 is the addition of a new 
#              import data style which is to import key=value pairs.  Basically, I added the ability to export data from 
#              AAAanalyze like this recently, but there are situations where you would then want to turn around and 
#              import the data in that format.  In particular, I'll be importing data to then fit to a direct binding
#              isotherm.  This also seems like a useful and general format, and I like it a bit better than XML/DOM, although
#              I should consider adding that at some point.
# 7/30/2012 SBP  A few additional changes have been made now.  Several bug fixes (PTI data is better handled, fixed Find_t50 etc.)
#	       Now I'm adding a few functions Stone had been interested in.  These are pretty small, but basically they allow
#              The fits to be exported as text (print_fit), the plots to use semilog and loglog axes.  

from math import *
import sys
import os
import string
import numpy as np
import scipy as sp
from scipy.optimize import curve_fit
import matplotlib as mpl
import matplotlib.pyplot as plt

# compbindeq is a solution for the single site competition binding equilibrium.  It can be used
# for both direct binding and competition binding.  The use of numpy (np) versions of sqrt pow and arccos
# was required to have it in a format that curve_fit would accept.  As it stands this is the general equation,
# but curve_fit needs it to be func(indepvar, paramstofit) so I then need to make a subfunction can be fit.
# At this point, it also looks like the parameters that are fixed need to somehow be passed, probably as a
# fixed variable.

#Solution to the competition binding equilibria adapted from:  Wang, Zhi-Xin, FEBS Letters v360 (1995) p111-114

def compbindeq(ligstar_tot, lig_tot, recpt_tot, K_ligstar, K_lig):
    terma = K_ligstar + K_lig + ligstar_tot + lig_tot - recpt_tot
    termb = K_lig*(ligstar_tot - recpt_tot) + K_ligstar*(lig_tot - recpt_tot) + K_lig*K_ligstar
    termc = -K_lig*K_ligstar*recpt_tot
    dum1 = np.sqrt((np.power(terma,2) - 3*termb)) 
    dum2 = (-2*np.power(terma,3) + 9*terma*termb - 27*termc)/(2*np.sqrt(np.power((np.power(terma,2) - 3*termb),3)))
    theta = np.arccos(dum2)
    recpt_free = -terma/3 + 2*dum1 / 3 * np.cos(theta/3)
    recpt_ligstar = (ligstar_tot*(2*dum1*np.cos(theta/3)-terma))/(3*K_ligstar+(2*dum1*np.cos(theta/3)-terma))
    recpt_lig = (lig_tot*(2*dum1*np.cos(theta/3)-terma))/(3*K_lig+(2*dum1*np.cos(theta/3)-terma))
    lig_free = lig_tot - recpt_lig
    ligstar_free = ligstar_tot - recpt_ligstar
    return ligstar_free, lig_free, recpt_free, recpt_ligstar, recpt_lig

#Bringing over a bunch of functions from AAAanalyze.v3b
# All of these need to be rewritten to deal with using numpy arrays as the main data object (I need that for
# using matplotlib and scipy) instead of simply lists.  In the process, I'll rewrite a few subtle aspects as well, 
# and add handles for the visualization of the data processing.

# find_max_inten function
# this finds the maximum intensity by chosing the five highest intensities, averaging their times.
# Then it picks the intensities in a window five points before and after (11 points total).  
# This somewhat elaborate process should minimize the effects of a single outlier intensity.  Reported is
# the average intensity in that window.  I could make this window flexible, but I won't for now.

# a difference here is that this function writes out some of its
# key info to a list of data that will be passed to the portion of the script that visualizes how barbed ends etc. are found.

# Adapting these functions to the new parameter model (all parameters passed, and pass_to_visproc is eliminated)

def find_max_inten(xv,yv,times_near_max=5, points_to_average=5):
    """find_max_inten(data) takes a [x-value] and [y-values] dataset and returns a best estimate for the maximum intensity """
    data1 = np.rec.fromarrays((xv,yv),dtype=[('time',float),('intensity',float)])
    proc_info = dict()
    data1.sort(order='intensity')
    est_of_time_for_max_inten = np.average(data1[data1.size-times_near_max:].time)
    data1.sort(order='time')
    time_near_max = 0
    for time in data1.time:
        if time < est_of_time_for_max_inten:
            time_near_max = time
    index_near_max = data1.time.tolist().index(time_near_max)
    intens_near_max = data1.intensity[index_near_max-points_to_average:index_near_max+points_to_average]
    times_near_max2 = data1.time[index_near_max-points_to_average:index_near_max+points_to_average]
    ave_max_inten = sum(intens_near_max)/len(intens_near_max)
    proc_info['estimated_max'] = ave_max_inten
    proc_info['points_to_calc_max'] = [times_near_max2,intens_near_max]
    return(ave_max_inten, proc_info)

# max_actin_poly_rate
# window refers to the width of the window used to measure the slope.  The number of points used to 
# calculate the slope is 2n+1 centered on the point of interest (the x-value the slope is assigned to).

def max_actin_poly_rate(xv,yv,total_actin=4.0,critical_conc=0.1,max_inten=1,min_inten=0, window=11):
    """max_actin_poly_rate(xvals, yvals, window) takes 1D numpy arrays for xvals and yvals, and an integer for window.""" 
    data1 = np.rec.fromarrays((xv,yv),dtype=[('time',float),('intensity',float)])
    proc_info = dict()
    mx_slope = 0
    index_of_mx = 0
    trend = [0,0]
    for line in range(window, data1.size - window):
        temp = np.polyfit(data1.time[line - window:line + window + 1], data1.intensity[line - window:line + window + 1],1)
        if (temp[0]>mx_slope):
            mx_slope = temp[0]
            index_of_mx = line
            trend = temp
    actin_per_inten_unit = (total_actin - critical_conc)/(max_inten - min_inten)
    value_of_max_actin_poly_rate = actin_per_inten_unit * mx_slope
    proc_info['trend_at_max_slope'] = [trend, 0.75*data1.time[index_of_mx], 1.25*data1.time[index_of_mx]]
    proc_info['points_to_calc_max_slope'] = [data1.time[index_of_mx - window:index_of_mx + window + 1], data1.intensity[index_of_mx - window:index_of_mx + window+ 1]]
    return(value_of_max_actin_poly_rate,proc_info)

# bes_at_max_actin_poly
# window and data are as in max_actin_poly_rate
# Of note here, I've only moved over a hybrid of the bes_at_max_actin_poly functions from AAAanalyze.V3b.  In this version, the max_inten is found using
# the find_max_inten function, which is a better solution than a simple call to max().  The max actin polymerization rate is still found using a call 
# to max() which isn't ideal at all.  The actual max value isn't much of a problem here, but the max will be very prone to noise in time of max.  
# Barbed ends need to know how much actin is present, and the noise in time of max is a huge problem.  I've toyed with using a parabola to get the center
# of the slopes, or using a centroid approach (which ought to be pretty good).  However, the take home message there was that the BEs_t50 was a better approach.
# This is mostly present for historical reasons.  I should try the centroid approach at some point.  

def bes_at_max_actin_poly(xv,yv,total_actin=4.0,kon=11.6,koff=1.3,critical_conc=0.1,max_inten=1,min_inten=0, window=11):
    """bes_at_max_actin_poly(xvals, yvals, **opts - [SBP need to edit this]) takes 1D numpy arrays for xvals and yvals, and an integer for window."""
    data1 = np.rec.fromarrays((xv,yv),dtype=[('time',float),('intensity',float)])
    proc_info = dict()
    mx_slope = 0
    index_of_mx = 0
    trend = [0,0]
    for line in range(window, data1.size - window):
        temp = np.polyfit(data1.time[line - window:line + window + 1], data1.intensity[line - window:line + window + 1],1)
        if (temp[0]>mx_slope):
            mx_slope = temp[0]
            index_of_mx = line
            trend = temp
    actin_per_inten_unit = (total_actin - critical_conc)/(max_inten - min_inten)
    value_of_max_actin_poly_rate = actin_per_inten_unit * mx_slope
    proc_info['trend_at_max_slope'] = [trend, 0.75*data1.time[index_of_mx], 1.25*data1.time[index_of_mx]]
    proc_info['points_to_calc_max_slope'] = [data1.time[index_of_mx - window:index_of_mx + window + 1], data1.intensity[index_of_mx - window:index_of_mx + window +1]]
    actin_at_max_poly_rate = total_actin - (actin_per_inten_unit * (data1.intensity[index_of_mx] - min_inten))
    value_of_bes_at_max_actin_poly = actin_per_inten_unit * mx_slope / ((kon * actin_at_max_poly_rate) - koff)
    return(value_of_bes_at_max_actin_poly,proc_info)

# find_t50
# window parameter here is the range of intensity used to find t50.  We pick the data which is in an 
# inten50 +/- window * (inten_max - inten_min) range, fit a line, find the value of time for which inten50 is 
# reached, then pick the timepoint before that.

def find_t50(xv, yv, max_inten=1, min_inten=0,window=0.05):
    """find_t50(xvals, yvals, max_inten, min_inten, window) takes 1D numpy arrays for xvals and yvals, and floats for max_inten, min_inten and window."""
    data1 = np.rec.fromarrays((xv,yv),dtype=[('time',float),('intensity',float)])
    proc_info = dict()
    inten50 = (max_inten + min_inten) / 2
    inten50_wind = window * (max_inten - min_inten)
    inten50range = [inten50 - inten50_wind, inten50 + inten50_wind] 
#    print "allowed inten50range: ",inten50range
    if window > 0.499:
        print "Find_t50 WARNING -- Specified value for Find_t50 window is atypical.  Usual values at 0.03 to 0.1, your value is:",window,".  This is the only parameter given to Find_t50 and is the fraction of intensity change above and below 50% to use in finding t50." 
    data1.sort(order='intensity')
    timesneart50 = []
    intensneart50 = []
    for index in range(data1.size):
        if data1.intensity[index] > inten50range[0]:
            if data1.intensity[index] < inten50range[1]:
                timesneart50.append(data1.time[index])
                intensneart50.append(data1.intensity[index])
#                print "Found point near t50",data1.time[index]," ",data1.intensity[index]
    lineneart50 = np.polyfit(timesneart50,intensneart50,1)
    t50a = (inten50 - lineneart50[1]) / lineneart50[0]
    t50b = 0
    data1.sort(order='time')
    for index in range(data1.size):
        if data1.time[index] < t50a:
            t50b = data1.time[index]
    proc_info['line_to_extrapolate_t50'] = [lineneart50, 0.75*t50a, 1.25*t50a]
    proc_info['points_to_calc_t50'] = [timesneart50,intensneart50]
    proc_info['t50b'] = t50b
    return(t50b,proc_info)

# find_BEs_t50
# This function actually does double duty.  It calculates both the barbed ends (BEs) at t50, and the actin polymerization rate at t50.
# it is assumed that total_actin, critical_conc, kon and koff are set appropriately outside of the function, using a global variable. Bad form, I know.
# In addition to the data (as xv and yv which are both numpy arrays) the function is passed window1, window2 and switcher.  The latter are: the window
# for calculating the slope (an integer, 2*window+1 points are used in the linear interpolation), the window for calculating t50 (as a fraction of total
# intensity change, a floating point number) and a switcher value which tells the function if we're calculating BEs or AP.

# Okay, a modification here.  Now I'm going to find t50, and find the slope at t50 in one operation.  Not sure why I didn't do that before.
# Basically, this means that window1 isn't used at all any more and this process ends up being a bit simpler.  I may want to reimplement the
# old somewhat baroque system at some point, but lets try this and see if I like it or not.

# def find_bes_at_t50(data, window1, window2, switcher):
def find_bes_at_t50(xv, yv, max_inten=1, min_inten=0,total_actin=4.0,critical_conc=0.1,kon=11.3,koff=1.6,window2=0.05, switcher=1):
    """find_bes_at_t50(xvals, yvals, window1,window2,switcher) takes 1D numpy arrays for xvals and yvals, and an integer for window, a float for window 2 and an integer for switcher."""
    data1 = np.rec.fromarrays((xv,yv),dtype=[('time',float),('intensity',float)])
    proc_info = dict()
    inten50 = (max_inten + min_inten) / 2
    inten50_wind = window2 * (max_inten - min_inten)
    inten50range = [inten50 - inten50_wind, inten50 + inten50_wind]
    data1.sort(order='intensity')
    timesneart50 = []
    intensneart50 = []
    for index in range(data1.size):
        if data1.intensity[index] > inten50range[0]:
            if data1.intensity[index] < inten50range[1]:
                timesneart50.append(data1.time[index])
                intensneart50.append(data1.intensity[index])
    lineneart50 = np.polyfit(timesneart50,intensneart50,1)
    t50a = (inten50 - lineneart50[1]) / lineneart50[0]
    t50b = 0
    data1.sort(order='time')
    for index in range(data1.size):
        if data1.time[index] < t50a:
            t50b = data1.time[index]
    proc_info['line_to_extrapolate_t50'] = [lineneart50, 0.75*t50a, 1.25*t50a]
    proc_info['points_to_calc_t50'] = [timesneart50,intensneart50]
    proc_info['t50'] = t50a
    actin_per_inten_unit = (total_actin - critical_conc) / (max_inten - min_inten)
    actin_poly_at_t50 = actin_per_inten_unit * lineneart50[0]  
    actin_at_t50 = total_actin - (actin_per_inten_unit * (np.polyval(lineneart50,t50a)-min_inten))
    value_of_bes_at_t50 = actin_poly_at_t50 / ((kon * actin_at_t50) - koff)
    if switcher == 1:
        return(value_of_bes_at_t50,proc_info)
    elif switcher == 2:
        return(actin_poly_at_t50,proc_info)
    else:
        return("huh? find_bes_at_t50 given the wrong switcher value")

# find_actin_nuc_rate is a function which takes an actin polymerization date set, and the relevant parameters, and outputs the initial rate
# of nucleation of new barbed ends.  This is accomplished by using the initial X% of the intensity change and fitting a parabola, which can be
# used to find the nucleation rate.  

def find_actin_nuc_rate(xv, yv, max_inten=1, min_inten=0,total_actin=4.0,critical_conc=0.1,kon=11.3,koff=1.6,inten_window=0.15):
    data1 = np.rec.fromarrays((xv,yv),dtype=[('time',float),('intensity',float)])
    proc_info = dict()
    actin_per_fluor_unit = (total_actin - critical_conc)/(max_inten - min_inten)
    threshold = min_inten + inten_window*(max_inten - min_inten)
    mask = data1.intensity.__lt__(threshold)
    fit_parabola = np.polyfit(data1.time[mask],data1.intensity[mask],2)
    t_para_max = (-fit_parabola[1] + np.sqrt(fit_parabola[1]*fit_parabola[1] - 4*fit_parabola[0]*(fit_parabola[2]-max_inten)))/(2*fit_parabola[0])
    proc_info['parabola_for_nuc_rate'] = [fit_parabola, 0, t_para_max]
    proc_info['points_to_calc_nuc_rate'] = [data1.time[mask],data1.intensity[mask]]
    nuc_rate = (2*actin_per_fluor_unit*fit_parabola[0])/(kon * total_actin)
    bes_at_t0 = actin_per_fluor_unit*fit_parabola[1]
    filamentactin_t0 = actin_per_fluor_unit*(fit_parabola[2]-min_inten)
    return(nuc_rate,bes_at_t0,filamentactin_t0,proc_info)


#Below I initialize the fixed variables that I'll use later.  Not sure I really need to give them values
#at this point, but I'd rather not run into a type error later.

pass_to_visproc = {}
fixed_lig_star_tot = 0.01
fix_lig_tot = 0.
fixed_recpt_tot = 0.5
fixed_K_lig = 1.
fixed_K_lig_star = 0.1

#We should load in the data.  I envision this as taking a data file, where each line is a either a 
#command or a data point.  The commands will be able to direct the program to load and fit one or more
#direct binding or competition binding data sets.  At this point I'll require that the data sets be in 
#the form of "data_point", processed conc, observable points.  I'll have to write another script to 
#reduce various observables into that format.  
#I also want a means of passing fixed parameters (eg. known Kds or fixed 
#reporter concentrations etc.)  Finally, I want a means fo telling the program to output text with the 
#fitting results, and to output the fits overlayed on the data to a file or the screen.

# Okay the above command now is really antiquated.  I'm now reading in a setup file.  Every line is 
# complete command with all the parameters that command needs.  Some of those commands can either 
# directly specify a data point or load in a data point for processing, or load in entire data sets.  

#comments:  Comments in the setupfile are indicated with a "#"
#commands:
#start_data_set datasetname
#   this begins a dataset with the name "datasetname"
#   this should be followed by a series of data points of the format:  conc   observable
#data_point   conc   observable
#   a data point to be incorporated into the active data set.  conc is the concentration of the varied ligand
#   obsrvable is the signal being followed.
#end_dataset
#   this will conclude the current data set.
#print_dataset
#   this prints the data set to standard output path
#fit_dataset mode params
#   this initiates a fit of the data.  Modes available:
#
#   direct_binding fits a titration of the receptor into the reporter
#       parameters that follow should be the reporter concentration, in uM, an initial guess for the Kd, and 
#          then the free and saturated observable values.  
#   direct_binding_fixed_obs fits a titration of the receptor into the reporter
#       in this case the parameters that follow should be the reporter concentration, in uM, an initial guess
#       for the Kd (in uM), then the free and saturated observable values.  The latter will not be fit.
#   direct_binding_fixed_free fits a titration of the receptor into the reporter
#       in this case the parameters that follow will be the reporter concentration, in uM, an initial guess
#       for the Kd (in uM), then the free observable value (which will be fixed) and a guess for the bound
#       observable values.
#   competition_binding fits a titration of competitor into a fixed amount of reporter and receptor
#       parameters that follow should be the reporter concentration, then the receptor concentration,
#       both in uM.  Next will be reporter Kd for the receptor in uM.  Finally, initial guess for the 
#       competitor ligand Kd for the receptor, the free reporter observable value and the bound observable
#       value.
#   competition_binding_fixed_obs fits a titration of competitor into a fixed ammount of reporter and receptor
#       the parameters and methods are the same as for competition_binding, with the exception that the
#       observable values are not free to fit, but fixed at the values provided.   
#          be optional, but must be specified in that order, such that to specify a fixed saturated 
#          observable, one must give an initial guess for the Kd and fix the free observable.      
#plot_data symbol
#   this adds a plot of the data to a figure.  This doesn't actually show the plot, that is done separately.
#   symbol can be chosen from the matplotlib symbol groups.  Examples are "ro" for red circles, "bs" for blue
#   squares, etc.  Leave off the quotation marks
#plot_fit symbol (range) (samples)
#   this adds a plot of the fit result to a figure.  You can optionally specify the range to be plotted
#   and how many samples within that range.  
#show_figure
#   this will show the current figure.  Multiple data sets and fits can be overlayed by using multiple plot
#   commands before the show/save figure command is used.  This clears the current figure.
#save_figure filename
#   this saves the figure to the given filename.  Format is specified by the extension.  I prefer .eps

# Need to update this to include the AAAanalyze functions.

# A tiny helper function which is used to see if a string can be turned into a single number.  Basically it returns
# 'False' if float(string) fails.

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

#The 'instructions' string is what gets printed if the user calls the script with 'help' or 'instructions'
instructions = "Instructions for fit_compbindingeq.py - ver1 SBP 2/1/2011\n\
What can the program do at this point?\n\
(1)  The program can read in several data types, all of them are text files at this point.  Addition of new data types is possible, including non-text files when you have a clear idea of how they are structured.  \n\
(2)  The program can perform a few routine data processing tasks.  The main ones that I use are fitting of binding and competition binding data, and analysis of actin polymerization data.  By its nature, addition of new tasks should be a fairly routine event.\n\
(3)  The program can overlay data sets and fits and display the resulting graphs, or write them out to a format that is editable in Adobe Illustrator.  \n\
(4)  The program is managed entirely from the commandline, using a script (SBP_data_analyze) and setup files.  While a bit different from how many of us deal with data handling, it means that doing the same thing ten times takes not a lot more time than doing it once.  Also, once you\’ve figured out how to do something, it will usually just be a matter of adapting a setup file to a similar task later on.\n\
Installing the program\n\
The program should run on Mac, PC and Linux, but I have only tested it on Mac at this point.  If you are a PC or Linux user let me know and we can work together to make sure it works.  \n\
To run the script you will need a specific version of Python and several add-on packages installed.\n\
These are: Python, Numpy, SciPy and Matplotlib.  At the moment, SciPy only works on python 2.6 and earlier, so you\’ll need to install versions of each at the Python 2.6 level. This is actually undersirable, Python2.6 is supposed to be phased out in a few years, v2.7 is the current supported version.  If SciPy ever makes a version for Python2.7 series, we should be able to adapt to those versions pretty quickly.  Versions for the 3.x series will be more work.  BTW- You can have multiple version of python installed on a single computer.  You\’ll notice in my examples that I call Python2.6 directly, this is because I have several version of Python installed.  \n\
At the moment (4/13/2011) this means installing (in order): \n\
Python 2.6.6 (http://www.python.org/download/releases/2.6.6/)\n\
Numpy for Python 2.6 (http://sourceforge.net/projects/numpy/files/)\n\
SciPy for Python 2.6 (http://sourceforge.net/projects/scipy/files/)\n\
Matplotlib for Python 2.6 (don\’t get the version for Python 2.7) (http://sourceforge.net/projects/matplotlib/files/matplotlib/matplotlib-1.0.1/)\n\
On Mac this means installing a \”Python Framework\”.  That should happen by default if you follow the above links and install the .dmg as you normally would.  Mostly, I mention this so the framework term sounds familiar when it comes up.\n\
Additionally, on PC you\’ll need some sort of command line system that you\’re comfortable with.  A lot of people like CygWin for this purpose.\n\
Running the program\n\
The program is just a script to be run in python.  To avoid problems with python version and pathnames, I typically run it as:\n\
python2.6 SBP_data_analyze.py setupfile.txt\n\
This means that I keep a copy of the version of SBP_data_analyze used for the analysis in the directory that the setupfile is in, and typically all the data used is also in the same directory or in subdirectories contained within it.  This has the advantage of keeping all the information of how the processing is done in one place.  As the script is still changing a fair bit, I like this approach of keeping the script used with the setupfile and data used.\n\
You will need a \’setupfile\’ to get anything done.  The setup file can have any name, but needs to be a simple text file.  It will simply be a text file that has one command per line.  Data_analyze simply steps through the setup file, running each command in the order that they appear, and it terminates at the end of the file.  In doing this it may read data, plot and display data, fit data and output information to the command line.  When I want to save the information output to the command, the command changes slightly.\n\
python2.6 SBP_data_analyze.py setupfile.txt > outputfile.txt\n\
There is also a special mode that doesn\’t require the setupfile.  This is the \’help\’ mode, which gives detailed instructions on how to use the script (basically this guide in text form). The instructions can be displayed as follows:\n\
python2.6 SBP_data_analyze.py instructions\n\
or\n\
python2.6 SBP_data_analyze.py help\n\
\n\
Major concepts\n\
There are a few major concepts that you need to know in using the script.  These concepts reflect specific design choices for the script that limit the scope of problems that it can handle, but make the program easier to use for the tasks it can handle.  The behavior of the program might be hard to follow if you aren\’t aware of these basic ideas.\n\
There is only one data set that is \’active\’ at any time.  \n\
When an operation processes, modifies or plots a dataset, there is only one data set to choose from.  This is the \’active\’ dataset.  This means that we can\’t do some advanced and global analyses that require multiple datasets.  However, most of the data processing that we do is only operating on one dataset at a time, so most of what I want to do is covered by this approach. Hopefully, this means that learning to use the script should be easier.  \n\
In some cases, many datasets are processed individually, but then we want to do something with the aggregated results.  In some cases, this script is still a fine means of handling the data.  The work around used in this case is that the script is run twice, with the output of the first run as the dataset used in the second run.  \n\
A new dataset is opened with the \’start_dataset\’ command.  We can\’t do any data processing until the dataset is complete though.  There are a variety of ways to enter data, but when we\’re done entering data, we use an \’end_dataset\’ command.  We can then do all the processing and plotting we want on that one data set.  \n\
When we\’re ready to move onto a new data set, we simply give a new \’start_dataset\’ command.  This opens a new dataset, but in doing so, the preceeding dataset is discarded.  \n\
Most setup file command lists will follow the following structure:\n\
start_dataset  (options...)\n\
(enter data)\n\
end_dataset\n\
(do something with the data)\n\
\n\
Setupfiles that deal with multiple datasets then repeat this structure until all the datasets have been handled.\n\
Only 2-dimensional, x-y pairs can be dealt with.  \n\
Of course this restricts the possible gamut of data and processing that we can perform, but most of what we work with can be distilled to x-y pairs.  When we only have x-y pairs to concern ourselves with, the syntax becomes fairly easy.  There is nothing inherently limiting us to x-y pairs in the python-numpy-scipy-matplotlib software that underlies the script by the way.  This is simply a choice of mine for this particular script.  When I find myself working with more complex datasets, I either write a custom script for the job, or work in another environment entirely (eg. Mathematica or Matlab).  This script is meant for use in the most common day to day work.\n\
There is only one figure being made at a time.\n\
There are several commands that produce graphical output.  Calling any of these commands will either start a new figure or will write graphical output to the existing figure.  The graphical output is accumulated, on the same axes, in the order that it is produced, until it is cleared out.  You should note that it isn\’t displayed while this is happening.  Display of the figure requires a deliberate command.\n\
In principal this is similar to the \’one-active-dataset\’ idea above.  We open a figure, we add stuff to it, we close the figure.  My usual work flow means that the further formatting of the figure happens in a different program entirely (usually Illustrator).  As a result the loop is a little different than it was for the dataset loop.\n\
While there are several ways to start a figure, there are two ways to clear it out \’show_figure\’ and \’clear_figure\’.  \’show_figure\’ produces a window on your screen with the accumuated figure, and clears out the figure information.  \’clear_figure\’ just clears the figure information, typically this is used in conjunction with \’save_figure\’.  This is because \’save_figure\’ will not clear out the figure data after writing its file.  Once the figure is cleared out, you may begin a new one, but in most cases, I make separate figures using separate setup files.  \n\
The setupfile commands\n\
Overview\n\
Each command takes up one line of a text file.  The line begins with a command name, which sets the mode for that line.  It is followed by a series of parameters which are specific to the command.  The command should come at the beginning of the line, and it is separated from the parameters by white space.  The nature and amount of white space is not important.  Several spaces is usually sufficient to make the separation clear when reading the setupfile, and works fine with the script.\n\
Importantly, each parameters must NOT have whitespace in it.  Underscores, dashes and the like are acceptible, but spaces and tabs cannot be put within a command or parameter entry.  Many commands have underscores in them, these are important.\n\
Making the setup file\n\
The setup file is a plain text file, at the moment ASCII text only.  If you aren\’t familiar with how to work with UNIX type text files this can sound a bit scary.  The simple answer is that you just need to use an old fashioned text editor and everything will be fine.  I favor VI (or vim) but there are lots of text editors out there.  On Mac a perfectly useable editor comes with OSX, it is called TextEdit, and you can find it in the Applications folder.  On PC, Notepad is a text editor.  You can find it in varioius places, but usually you can launch it by using windowbutton-R, and then typing \’notepad\’.\n\
\”#\”\n\
The simplest command.  Beginning a line with \’#\’ simply designates the line as a \”comment\”.  The script simply ignores the line altogether.  This allows you to explain what you are doing in you setup files, and to quickly remove lines from processing, without actually deleting them.  The only thing that the script does do with a comment is that it tracks the line number.  When the script reports line numbers, it will include any comments in the numbering.  This allows you to jump to the appropriate line number in the setupfile when attempting to repair an error.\n\
\” \”\n\
The behavior of the script in response to an empty line is slightly different than for a comment.  When the script finds an empty line, it also skips a line in the output.  In some cases this can be helpful for diagnosing problems (\”well, it skipped fifteen lines before breaking, so the problem comes after that.\”)\n\
\”start_dataset\”\n\
This begins a new dataset, and discards any existing data.  This command must be called before entering data.  There are several parameters that should be passed along with \”start_dataset\”.  These are (in the order that they appear):\n\
data_set_name    x-axis_label    y-axis_label\n\
\”end_dataset\”\n\
This closes the data set to further addition, and should be used to split up the data entry step from any analysis steps.  In addition it performs a few formatting checks and repairs, which means that if you try to process data further before this command is used, it may work sometimes, but not always. \n\
 \”data_point\”\n\
This command allows the direct entry of a single data point.  The entered data point will be added to the end of the current dataset.  It has two parameters, the x-value and the y-value.  Thus the command should be called as:\n\
data_point   x-value   y-value\n\
for example\n\
data_point   10   25000\n\
\n\
\”change_data_dir\”\n\
This command allows changing the directory used to find data files while the script is running.  \’change_data_dir\’ is called with up to two parameters.  The first is the daughter directory to enter from the directory that the script is invoked in.  The second is an optional parameter \”quiet\” for times when you don\’t want it chattering about all the directory changes that it has made.  For example:\n\
change_data_dir   110414_data   quiet\n\
An important note:  the behavior of \’change_data_dir\’ isn\’t the same as a traditional change directory commands.  Instead, it begins in the directory from which the script is called.  It will then descend into a daughter directory from that parent directory.  The odd behavior is that this is true every time it is called, such that there is no capacity to go up in directory, and the change of directory occurs from the original parent directory every time.   \n\
\”data_file\”\n\
This is a command with several sub-modes.  It allows importing in an entire data set from an existing file.  There are several sub-modes that allow the importation of different kinds of data in different formats.  These are explained below.  Generally these commands will have the format of:  \n\
data_file    data_type    data_file_type    data_file_name   parameters\n\
Several data types are supported, and more can be added if there is of interest.  The acceptable values for data type are currently:  anisotropy, intensity, and intensity_timecourse, marked_up_data.  The acceptable values for data_file_type are currently:  raw_text, generic_text, PTI_aniso_text, PTI_inten_text_raw, PTI_inten_text_corr, and key_value_pairs.  It is notable that the data type and data file type aren\’t strictly paired up.  If you use a data file type which doesn\’t make sense for the data type, you will usually raise and error, but not always.  Typically, there is some sensible data_file_type for each kind of data, and each kind of analysis. I have not exhaustively tried all the possible combinations, if you use odd combinantions, you may get odd results.\n\
data file types:\n\
There are several data_file_types currently supported.  I\’ll note here that all of these are simple text files, with some sort of line by line structure.  There is no reason that we have to support only flat text files as the script develops, but there hasn\’t been a lot of call for otherwise so far.  If there is an obvious format that is desired, we can try and set it up. \n\
generic_text or unknown type:  The default data type is just x-value/y-value pairs, separated by white space, with pairs on separate lines, and no header information or trailing information.\n\
PTI_aniso_text:  A PTI format of data, with header information, followed by four columns of data.  The columns are time, detector-1-inten, time, detector-2-inten.  On reading this format, the script drops the initial header information, and assigns x temporarily to the second column, and y temporarily to the fourth column.  Used in conjunction with anisotropy calculations, this information is further processed before appending to the growing dataset.\n\
PTI_inten_text_raw:  A PTI format of data (written to text), with header information and two columns of data, time intensity pairs.  Time is temporarily treated as x-values, and intensity is temporarily treated as y-values.  \n\
PTI_inten_text_corr:  Similar to PTI_inten_text_raw, but with four columns.  The third and fourth columns are another time column, and the \’corrected\’ intensity.  In this case only the corrected intensity is kept.\n\
PTI_inten_text_corr_but_use_raw:  Similar to the above two formats.  There are four columns in the data (after the header), but the second column is kept as the y-values.\n\
key_value_pairs:  A flexible format in which each data point is a line with key=value pairs separated by white space as the data point.  The data point can then contain more than two values.  The actual information on each line can be in any order, but each line must contain a key that matches the x-key and y-key.  These are given after the data_file in the command.  For example (all on one line though):\n\
data_file  marked_up_data  key_value_pairs  110414_processed_data  time  velocity\n\
The power of this last data file type is that a lot of information can be associated with each data point.  I have been favoring its use with processed data.  One can include information such as the original data file used, how it was processed, who collected it, when it was collected, etc. in the data format, but then ignore all that when it comes to down stream processing.\n\
data_types:  The data_type and data_file_type aren\’t strictly coupled.  Ideally, one can take a kind of data in a variety of formats, and in the process of loading it convert it into the common data type to go forward.  As a result it is more important to know what the data type means in terms of processing.\n\
anisotropy\n\
At the moment the anisotropy data_type supports PTI_aniso_text and generic_text data_file_types.  The file should give two channel intensity information for a single piece of anisotropy data, possibly as a time series that will be averaged into one data point.  In addition to the file information (data file type and file name), data on the blank, G-factor and G-blank will also be needed.  Further, the x-value to associate with the resulting anisotropy is needed.  These last two points are handled similarly.  As the last parameter, the word \’blank\’, \’g-blank\’, \’g-factor\’ or the x-value should be passed.  The \’blank\’ keyword tells the script that these intensities should be subtracted from each anisotropy point.  The \’g-blank\’ keyword tells the script that the intensities should be subtracted when calculating the anisotropy.  The \’g-factor\’ keyword tells the software that this data is useful for calculating the g-factor for anisotropy calculations. For example:\n\
data_file   anisotropy   PTI_aniso_text   110414_100nMVCA.txt   100\n\
or\n\
data_file  anisotropy   generic_data   110414_bufferblank.txt   blank\n\
If you try to load a data point prior to loading the blank, g-blank or g-factor data, an error will be raised.  Further, if you load those, then some data, then a new blank, g-blank or g-factor, only the most recent will be used for later data points.  Finally, the blank, g-blank and g-factor information is _not_ cleared out when a new data set is created.  If the script can find anything for those values, it will use them.   \n\
\n\
intensity\n\
Intensity data files will be treated a bit like the anisotropy files, but they aren\’t yet fully supported.  It will take the file, average it and possibly subtract off a blank value.  \n\
intensity_timecourse\n\
This takes a file that contains time intensity pairs, and loads it into the x-y data (time is x, intensity is y).  Currently, flat time-intensity pairs are supported (generic_text), PTI_inten_text_raw and PTI_inten_text_corr are supported.  Others should be easy to add, let me know what is needed.\n\
Additionally, the intensity_timecourse allows for the inclusion of a deadtime.  These are good to collect as a matter of general practice, I recommend it.  Sometimes you need them, sometimes not, but they are easy to record at the time and hard to realistically estimate after the fact.  If a number is given following the data_file, it is used as the deadtime, in whatever units the time values are given in.  For example:\n\
  data_file   intensity_timecourse   PTI_inten_text_raw   110414_100nMVCA.txt   25\n\
\n\
marked_up_data\n\
marked_up_data is meant to be a general class of more advanced data types, but we\’re beginning small.  (I\’d like to add XML at some point if anyone feels up for it).  As it stands we\’re left using marked up data only with key_value_pairs data file types.  When marked_up_data is used, it uses as specific x-key and y-key to figure out which data to use.  See the example in the key_value_pairs data_file_type section.\n\
\”print_dataset\”\n\
This prints the current x-y pair data set out to the commandline (or a file if you\’ve redirected it).  This allows including the processed form of the data in any log files which are stored, or capture of the output data for use in other applications.  Data is output as:\n\
x-value1    y-value1\n\
x-value2    y-value2\n\
x-value3    y-value3\n\
... etc.\n\
\n\
\”fit_dataset\”\n\
This command allows nonlinear regression of the current data set by models specified in the script.  Currently only binding isotherms and competition binding isotherms are set up, but any function with an even reasonably compact form (ie. full solution of the competition binding isotherm) can be given in the script.  \n\
fit_dataset has several \”submodes\” each corresponding to a different model to fit to.  Currently these are:  direct_binding, direct_binding_fixed_obs, direct_binding_fixed_free, competition_binding, and competition_binding_fixed_obs.  I plan on adding more as I go forward (exponential decays are high on this list).\n\
direct_binding\n\
This fits the data to a direct binding isotherm.  The x-value is hardcoded as the concentration of the receptor, and the reporter concentration is given as a parameter (which is fixed).  Following the direct_binding submode parameter, one should pass in order:  the reporter concentration (in concentration units that match the x-data), then initial guesses for the Kd of the reporter-ligand interaction, the observable in the free state and the observable in the bound state.  For example in an anisotropy detected binding curve (in uM units) we might have:\n\
fit_dataset  direct_binding  0.01  1  0.05   0.25 \n\
For a fitting of 10nM of reporter, initial guess of 1 uM for the Kd, 0.05 for the free anisotropy and 0.25 for the bound anisotropy.\n\
direct_binding_fixed_obs and direct_binding_fixed_free\n\
These have exactly the same usage as direct_binding, but the initial guess for both observables become fixed in direct_binding_fixed_obs and the initial guess for the free value of the observable becomes fixed in the direct_binding_fixed_free mode.  Typically, one can make a very good estimate of the free observable (ie. the 0 receptor point) and thus fitting it isn\’t necessary in many cases.  \n\
competition_binding\n\
This fits the data to a competition binding isotherm.  It is assumed that the experiment is setup with fixed reporter and receptor, and that a competiting ligand is titrated in.  The x-data is to be the competitor concentration.  The command is called with several parameters.  These are (in order):  the reporter concentration, the receptor concentration, Kd for the reporter-receptor interaction, then initial guesses for the Kd of the receptor-competitor interaction, the reporter observable in the free state and the observable in the bound state.  For example (assuming units of uM):\n\
fit_dataset  competition_binding   0.01  0.5  0.125   0.5  0.05  0.25\n\
Would fit the data to a competition binding isotherm with 10nM of reporter, 500nM of receptor, 125nM Kd for the reporter-receptor interaction and initial guesses of the receptor-competitor Kd of 0.5uM, the observable in the free state of 0.05 and the observable in the bound state of 0.25.\n\
competition_binding_fixed_obs\n\
This is the same as competition_binding, but the observable values are not fit, they are set and left alone.  Otherwise the meaning and the order of the parameters are the same as for competition_binding.  When observables can be reasonably estimated, this often improves fitting confidence substantially.  When small changes are observed, fits are reasonable, but if very large differences between the observables found in a direct fit and a competition fit are found, consider the fitting model carefully.  This can be a sign of interactions between the competitor and the reporter, or that the 1-1 model assumed here is wrong.\n\
Plotting commands:\n\
\”set_plot_limits\”\n\
This command restricts the range of data and fit shown to the specified values, as opposed to the default behavior which is to display the entire data range.  The behavior of the command is a little odd (I may change this later).  If one parameter is given, it sets the maximum x-value displayed.  If two parameters are given, they set the minimum and maximum values for the x-axis.  If four values are given then it sets the minimum and maximum values for the x-axis, followed by the minimum and maximum values for the y-axis.\n\
\”plot_data\”\n\
This plots the current data set to the accumulating figure.  The behavior can be simple, or more complicated.  At its simplest, plot_data is called with a symbol parameter.  For example:\n\
plot_data  ro\n\
The symbol parameter is composed of a letter for the color and a letter (or two) for the symbol/line shape.  There is a fairly extensive series of options here.  These are all drawn from the matplotlib plot command directly, I refer you there for complete details.  However, here is a summary of commonly used values.\n\
colors:  r = red, b = blue, g = green, c = cyan, y = yellow, m = magenta, k = black\n\
symbols:  o = circles, s = squares, v = triangle down, ^ = triangle up, * = star, + = plus mark, x = and x and D = diamond\n\
line styles:  - = lust a line, : = dotted line, -- = dashed line, -. = dot-dashed line\n\
symbols and linestyles can both be specified.  For example:\n\
plot_data  bs-\n\
gives blue squares marking the data, with a blue line connecting them.\n\
By default the name you gave to the data set when the start_dataset command was given is used in any legend.  Each plot_data command puts an entry into the legend, unless otherwise stated.  \n\
The plot_data command interfaces with the pyplot.plot command of matplotlib (this is why we had to install it).  This actually is a pretty flexible system and many of its more advanced modes can be accessed through plot_data in the verbose/detailed/explicit mode.  This mode is accessed by passing \”verbose\” as the first parameter to plot_data.  The remaining parameters should then be a series of key-value pairs of the format key=value.  In this way you can specify lot\’s of options that we don\’t ordinarily have access to.  There are too many to list here, but I\’ll give a quick block that I use fairly often which shows a fair bit of the scope of options.\n\
plot_data  verbose  linestyle=-  linewidth=0.1  color=b  label=_nolegend_\n\
\n\
plot_data  verbose  marker=^  color=b   markevery=40 markersize=9\n\
In the first command we specify a few details about the line to be used, and then give the \’label\’ as _nolegend_.  This means this command does not get a line in the legend.  In the second command markevery=40 means that only every 40th data point gets a symbol.  Used together, these give you a thin blue line which indicates every data point, and a blue triangle (pointing up) every 40th point, and only one entry in the legend.  If one desired to do so, one could also use the label=\”new_name\” if you wanted to give a different name to the dataset at this point.  However, if you do this, recall that whitespace in a parameter is a problem, and use of a space will result in an error.\n\
\”plot_fit\”\n\
plot_fit plots the current fit to the current figure.   Just giving a command plot_fit with no parameters works, and will produce a black line that spans the same x-range as the data that it is fitting.  If it is called with one parameter, then the line marker can be specified (limited to simple plot_data type symbol parameters, eg.  r- for a red line).  If three parameters are provided, they are the symbol, then the minimum x-value for the fit, then the maximum x-value for the fit.  If four parameters are provided, the last one specifies how many points to sample the fit at (default is 200), this will control how smooth the curve looks.  At the moment the fit will not get an entry in the legend.\n\
\”add_legend\”\n\
add_legend adds a legend to the figure.  It includes a line for each call to plot_data (unless this is overridden with a label=_nolegend_ verbose option, see above).  The legend is placed where the software deems the best location is for the legend.  It does this by trying to minimize how much data is obscured by the legend.  In some cases there is no good solution for the legend placement within the area of the axes.   At the moment I don\’t have a good solution for this in the script, locating the legend outside of the axes is not easy to do in matplotlib.  However, if you save the data as an .eps, the data is still there if you move the legend out of the way in Illustrator or related software. \n\
\”show_figure\”\n\
Up until this point you\’ve been making calls to plot_data etc. and no chart has shown up.  show_figure causes two things to happen.  First, a chart actually shows up in a separate window.  The second thing that happens is that the current figure is cleared when the figure is shown.  In principle, one can save the chart from that window, but I recommend against that (use save_figure instead).  When you\’re done looking at it the chart can be dismissed by closing the window.  Importantly, the script stops what it is doing until the window is closed.  I\’ve had trouble with python crashing if I change windows a bunch while one of the figures is being shown, so I\’d suggest inspecting it, and closing it before moving on to anything else.  Remember, you should be able to get it back easily by just rerunning the script.\n\
\”save_figure\”\n\
This command saves the figure to a file.  The file saved to will be in the current directory (if you\’ve changed directory using change_data_dir, I believe the destination directory changes here as well), with a filename specified by the only parameter for this command.  Of note, the filename should include an extension (a \”.\” and three characters).  The save_figure command figures out what format to use from this extension.  Many file types are supported (.pdf, .jpg etc), I strongly recommend that you use \”.eps\” as it opens sanely in Illustrator and allows manipulation of the vector based information there for conversion into nice figures later.\n\
\”clear_figure\”\n\
This command clears out all the current figure information.  This primarily gets used when you want to create and save a figure without ever showing the figure.  By using save_figure xxx followed by clear_figure, a single setupfile can create many figures in one shot without them being displayed on the screen.  \n\
\”meta_data\”\n\
The meta_data command allows you to place some annotation information into the setupfile, and have the program read the annotation.  At the moment this doesn\’t cause much to happen, but it may get expanded in the future.  Principal use is that AAAanalyze can be told to output it\’s analysis in a format consistent with the meta_data format, and will append any provided meta_data with its output.  Meta_data is passed to the script as an arbitrary number of parameters formatted as key=value pairs.  For example:\n\
meta_data  date=04/14/2011  filename=110414_100nMVCA.txt   processedby=SBP   concentration=100  material=VCA\n\
This has been quite helpful in setting up other scripts for summarizing processed data, and for generally making it a bit clearer what each line is talking about.  \n\
\”print_meta_data\”\n\
Sometime you want the meta_data to appear in the output log file.  You need to tell the command what format to use for the output data, at the moment, only key_value_pairs are supported.\n\
\”AAAanalyze\”\n\
This command analyzes actin assembly assays (hence the name).  It is a rewriting of the older AAAanalyze script, taking advantage of numpy, scipy and matplotlib along the way.  It adds a lot functionality and, in a few key places, changes functionality from that of the older script.  It is designed to take an actin assembly assay, find the maximum intensity, find the minimum intensity, find t50, actin polymerization rates, calculate barbed ends, and calculate nucleation rates.  There should be a bit of user supervision over this task, but for the most part the software should reasonably automate the process.\n\
There are many submodes for AAAanalyze.  They pertain to different analysis methods.  In a given setupfile, one could certainly analyze the same data multiple ways if desired.  The available analysis (sub-) modes are:\n\
 Find_BE_t50\n\
This finds the barbed ends at the time at which half of the observable change has occurred.  I favor this method over finding barbed ends at max actin polymerization rate, or max barbed ends, because it eliminates an avoidable source of error.  Find_BE_t50 needs several additional parameters.  First it needs the total actin concentration (typically in units of uM, but it does not need to be this way).  Next the barbed end elongation rate constant, kon is given, also in concentration units that should match the total actin concentation, and time units that match the x-data.  After that is the barbed end elongation off-rate constant, koff (again, matching the time units of the x-data).  Finally, a window parameter is given.  This is the fraction of change in intensity used for calculating the slope at t50.  The slope calculation is very sensitive to noise, and thus this window should be increased or decreased depending on the noise level, 0.03 to 0.08 should work in most cases (0.06 is a favorite for the plate reader, the PTI and JY data should be able to be processed with narrower windows).\n\
BarbedEnds_maxAP\n\
This finds the barbed ends at the time of the maximum actin polymerization rate.  With one exception, the parameters are the same as for Find_BE_t50.  The window for finding slopes is defined differently, here it is the number of points before and after a given time that should be included in calculating the slope at the given time.  The width of the window will be a function of how fast the data is changing in time, and of the noise level.  In many cases, a value for the window that works well for a fast polymerization run, will work poorly for a slow assembly reaction.  Start with 7, adjust up or down as necessary.\n\
Find_max_AP\n\
This finds the maximum actin polymerization rate, with parameters that match BarbedEnds_maxAP.  This command doesn\’t actually use the kon and koff values for anything other than calculating the critical concentration.  As a result if you don\’t know these under you conditions, but do know the critical concentration, just make sure their ratio is correct for your critical concentration.  The window is also defined as it was in BarbedEnds_maxAP, which is to say it is an integer, and 7 is a good place to start.\n\
Find_t50\n\
This mode finds the time at which half of the observed change has occurred.  There is one parameter for this calculation, a window that is defined as the fraction of the data above and below the halfway point in the intensity change, which is used to calculate the time at which half of the change has occurred, t50.  There are in fact several tricks which get used here to minimize errors associated with random error.  In particular, t50 is found by taking several points around the half intensity, and using linear regression to fit a line through them, and then using this line to find t50 instead of the data directly.  This assumes that the data is well approximated by a line in this local region, but my experience finds this to be true.\n\
Find_AP_t50\n\
This mode finds the actin polymerization rate at t50.  It needs four parameters, which match Find_BE_t50, the total actin concentration, kon, koff and a window for calculating the slope at t50.  The window is a real number between 0 and 0.5.  Usually something the range of 0.03 and 0.08 is sensible, with 0.06 being a good starting point.  Formally, kon and koff aren\’t needed for this calculation, except to determine the critical concentration.  Thus, their values are not essential, except that their ratio should be a good estimate of the critical concentration.\n\
Find_NucRate\n\
This finds the nucleation rate during the initial portion of the kinetic.  The nucleation rate is defined as the rate at which barbed ends are generated as a function of time, and when that is constant, the nucleation rate calculation makes sense.  Note, this doesn\’t need to be constant, but in many cases is pretty close.  Parameters passed are the total actin concentration, kon, koff and the intensity window to use for the calculation.  0.1 is a good place to start.\n\
override\n\
Sometimes data isn\’t perfect, but it is still usable.  Perhaps there is a big dust blip, or a noise spike causes the maximum to be found in a weird place.  In some cases we can reasonably come up with a way to exclude a portion of the data, or to force a parameter to some value, such that the data set is still useful.  As a result there is an override mode.  \’override\’ should be given as the first parameter instead of the usual submode setting.  The submode and its parameters follow, and an arbitrary number of overrides are given as key=value pairs as the last parameters for the command.  For example: \n\
Overrides available:  mask_data (allows the analysis to ignore a portion of the data), min_inten (forces the minimum intensity to a fixed value), max_inten (forces the maximum intensity to a fixed value), min_window (which allows for more than one point to be used to find the minimum intensity), output_format (output_format=key_value_pairs, provides a means of formatting the AAAanalyze output information in key=value pairs, which can then be read in in a separate instance of the script or by other scripts).\n\
\”show_analysis\”\n\
Show_analysis shows the data associated with an AAAanalysis processing of one data set, with several calculated features identified on the data.  The exact features shown will depend on whether overrides are used, and on the mode of AAAanalysis used.  I strongly recommend using this, most problems with the analysis can be identified easily in this way, and usually corrected.\n"

numcommandlineargs = len(sys.argv)
if numcommandlineargs == 2:
    setupfile = sys.argv[1]
    if setupfile == "help":
        print instructions
        sys.exit()
    elif setupfile == "instructions":
        print instructions
        sys.exit()
    else:
        print "input file is: ", setupfile
        starting_directory = os.getcwd()
        print "working directory is: ", starting_directory
        if os.access(setupfile,os.R_OK):
            pass
        else:
            print "Can't access file:   ", setupfile, "   Exiting!"
            sys.exit()
        setup_file_stream = open(setupfile,'r')
        setup_file_instructions = setup_file_stream.readlines()

# It should be noted here that the observable can be anything that linearly reports on the concentration of 
# the reporter in solution.  I'm envisioning fluorescence anisotropy of the reporter, but it could really be
# anything.

#We've split the setupfile into a series of instructions.  I'll execute them top to bottom now.

# first we initialize a few variables outside of the main loop.  These get used later, and need an initial 
# value.  
accepting_data = 0
data_type = "intensity"
line_counter = 0

# okay, begin main loop.  We now march through the instructions one by one and deal with them.
# It is important to realize that any command must occupy only one line.  Commands can be set up to 
# inherit information from prior commands, but any command can only take up one line.

# The command (ie line) is taken in, and split up by white space.  Spaces or tabs work, and multiple
# spaces or tabs are treated as a single separator.
# once split up, the first item in the command sets the mode, subsequent items are the parameters and
# are specific to the mode used.
for num in range(len(setup_file_instructions)):
# The line_counter just tracks where we are in the setupfile in counting numbers of lines.  
# This gets used in most error responses (eg. "there a mistake of this type on line# 12")
    line_counter = num + 1
# Get the current command as a line
    workingline = setup_file_instructions[num]
# split the line into items
    working_command = workingline.split()
# this just allows up to skip blank lines.  A blank line shows up in the output, allowing one to 
# either format the output a little, or just keep track of what is happening a little.
    if working_command == []:
        print
        continue
# Here we assign 'working_mode' to be the first element
# and 'working_params' are the rest of the items.  Note, an item cannot span whitespace, so every interpretation
# below must not have whitespace in a given item.  A space in a filename will kill you.  Don't use them.
    working_mode = working_command.pop(0)
    working_params = working_command

# This will allow commenting out of lines by starting it with a '#'.  Only whole line comments are allowed 
# at present.
    if working_mode[0] == "#":
        continue

# Now for each command, we match the first element to a list of modes, and pass the mode the parameters.

# The start_dataset mode simple opens an empty x,y dataset.  It also retrieves the datasetname (first param)
# the x-axis label (second param) and the y-axis label (third param).  
# A final note on the design of this whole program.  I want to keep it simple.  Thus we're only treating 
# x,y data and only one dataset or fit is possible at a time.  Thus when you open a new dataset, you throw
# out any existing dataset or fit results that you might have.  Any plots made persist until show_figure or save_figure
# are called.  

    if working_mode == "start_dataset":
        accepting_data = 1
        have_fit = 0
        pass_to_visproc = {}
        metadata = {}
        x_data = []
        y_data = []
        datasetname = working_params[0]
        if len(working_params) == 3:
            xaxislabel = working_params[1]
            yaxislabel = working_params[2]

# The end_dataset mode closes the data set.  This means no further data will be added at that point.  This makes the 
# program a little more rigid in its behavior.  Either you're creating a data set, or you're doing something with a 
# complete data set. There is no fitting or plotting partial data sets.  If one wants to plot a partial data set and 
# later a complete data set, this is best done in two completely separate operations.  
# A final point on end_dataset, this is also a point where we make sure the data is stored in np.array's.  These are
# needed later, so it seems best to force them at this point.
    elif working_mode == "end_dataset":
        accepting_data = 0
        x_data = np.fromiter(x_data, float)
        y_data = np.fromiter(y_data, float)

# The data_point mode is the simplest way to enter data, it is typed into the setupfile directly and
# read from there into the dataset one data point at a time.  Again we're rigidly using only x,y, data here.
# Thus, if there are more parameters with data_point, they will simply be lost. 
    elif working_mode == "data_point":
        if accepting_data == 1:
            x_data.append(string.atof(working_params[0]))
            y_data.append(string.atof(working_params[1]))
        else:
            print "At line# ",line_counter, " tried to add data without an open dataset."

# Working mode of change_data_dir allows one to change what directory you are drawing data from during the
# execution of a single setupfile.  You should note how this is implemented.  It is assumed that you've 
# run the script from a directory above all the directories your data are in.  If you data are in the 
# directory you run the script from you never have to change directories.  But to call data from multiple
# start one layer above and you can go one layer down, get data, then change to another directory, one
# layer down from the start.  This is notably different from the usual way changing driectories is done,
# but seems consistent with what is expected of the program.  I may add a more capable directory 
# changing method later, but this will remain the default behavior.

# More on change_data_dir...
# now with "quite" mode.  Pass it 'quiet' as the second parameter (after the directory to go to)
# and it won't tell you what it is doing.  Possibly a useful tool when worrying about the format of the output 

# Note to self, this could probably be more elegantly crafted with a "try:" statement.
    elif working_mode == "change_data_dir":
        os.chdir(starting_directory)
        os.chdir(working_params[0])
        if len(working_params) == 2:
            if working_params[1] == "quiet":
                pass
            else:
                print "working directory changed to: ", starting_directory + "/" + working_params[0]
        else: 
            print "working directory changed to: ", starting_directory + "/" + working_params[0]

# Working_mode of data_file.  This is one of the modes that has sub-modes.  Basically, there are different
# reasons one might want to read data from a file, and the behavior that results will be quite different.  
# For some reason I've decided to allow data_type and data_file_type to be specified independent of 
# one another.  
    elif working_mode == "data_file":
# check to make sure the dataset is 'open'
        if accepting_data == 1:
# Wow, this could be way better written using an 'in' statement.  Rewrite later.  Basically, the following
# just checks to see if working_params[0] matches an acceptible data_type, and then either 
# complains if it doesn't recognize it (and assumes whatever you did last remains true) or assigns data_type to 
# the value of working_params[0] if it does recognize it.  Again a single if ... in should handle that.
            if working_params[0] == "anisotropy":
                data_type = "anisotropy"
            elif working_params[0] == "intensity":
                data_type = "intensity"
            elif working_params[0] == "intensity_timecourse":
                data_type = "intensity_timecourse"
            elif working_params[0] in ('MarkedUpData','Marked_up_data','Marked_Up_Data','markedupdata','mud','MUD','marked_up_data'):
                data_type = "marked_up_data"
            else:
                print "At line# ", line_counter, " unrecognized data type given, treating as ",data_type
# set default data_file type.
            data_file_type = "generic_text"
# again we check to see if the value in working_params[1] is supported.  Again, I could ahve written this more 
# elegantly.  A single if ... in ... call would suffice here.
            if working_params[1] == "raw_text":
                data_file_type = "raw_text"
            elif working_params[1] == "PTI_aniso_text":
                data_file_type = "PTI_aniso_text"
            elif working_params[1] == "PTI_inten_text_raw":
                data_file_type = "PTI_inten_text"
            elif working_params[1] == "PTI_inten_text_corr":
                data_file_type = "PTI_inten_text_corr"
            elif working_params[1] in ('key_value_pairs','KVP','kvp','Key=Value','key=value','Key_Value_Pairs','keyvaluepairs','KeyValuePairs'):
                data_file_type = "KeyValuePairs"
# and if the data_file_type doesn't match a supported one, assume it is 'generic_text' or flat x   y pairs.
            else:
                pass
# get the filename for the data_file to workfrom from workingparams[2]
            data_file = working_params[2]
# check to see if we can access the file
            if os.access(data_file,os.R_OK):
                pass
            else:
                print "At line number: ", line_counter, " can't access file:  ",data_file
                sys.exit()
# begin specific behavior for 'anisotropy' data.  Key thing to realize here is that anisotropy
# data means that we're reading in two channel intensity data and converting it to fluorescence anisotropy.
# This means that there is a few different kinds of data.  First is an actual data point.  These will be V,H intensity
# pairs.  There can be multiple samplings in a single file, but these will be averaged together into a single data point.
# The anisotropy data point will output fluorescence anisotropy, but needs an x-value, this is given as the fourth 
# parameter, and corresponds to whatever the independent variable will be later.  Second is a blank, designated be the 
# fourth parameter being given as blank.  Third is the blank for the G-factor calculation, designated with g-blank. 
# Fourth, is the intensity for the G-factor calculation itself.  This is designated by g-factor in the fourth parameter.
#
# A few somewhat odd behaviors designed in.  When a dataset is opened, or closed, the blank, g-blank and g-factor are
# not modified.  Thus, we can have these pieces of information can persist from one data set to another.  This is
# relevant when data you want to fit separately was collected such that it shares the same blanks etc.  for clarity
# though I'd recommend resetting these values with each dataset. 
            if data_type == "anisotropy":
# A bit about 'PTI_aniso_text'.  This file type results when a two channel V, H short timecourse is copied from Felix
# and pasted into a text file.  We're assuming that the correction factor is not applied.  This just sets the number of
# lines to skip at the beginning, and which columns to assign to x-data and y-data. 
                if data_file_type == "PTI_aniso_text":
                    time1, dumx, time2, dumy = np.loadtxt(data_file, skiprows=4,unpack=True)
# generic text is just x,y pairs with no header or trailing information.  Sortof the default approach.
                elif data_file_type == "generic_text":
                    dumx, dumy = np.loadtxt(data_file, unpack=True)
                else:
                    print "Something is wrong with choice of data file type.  Line# ", line_counter
# I've been using a real simple set of data import routines here, thus if they haven't successfully been turned
# into floating point numbers, we'll have problems with the next step.
                dumx = np.mean(dumx)
                dumy = np.mean(dumy)
# check to make sure the fourth parameter is given.  
                if len(working_params) < 3:
                    print "Anisotropy called without value declared, line# ",line_counter
                    continue
# if the fourth parameter is given and it is a number, then this is a datapoint.  Calculate anisotropy.
                elif is_number(working_params[3]):
                    if is_number(blank_inten[0]) and is_number(blank_inten[1]) and is_number(g_blank_inten[0]) and is_number(g_blank_inten[1]) and is_number(g_inten[0]) and is_number(g_inten[1]):
                        g_factor = (g_inten[0]-g_blank_inten[0])/(g_inten[1]-g_blank_inten[1])
                        aniso = ((dumx - blank_inten[0]) - g_factor*(dumy - blank_inten[1]))/((dumx - blank_inten[0]) + 2*g_factor*(dumy - blank_inten[1]))
# the number in the fourth parameter gets appended onto the x-data, the anisotropy onto the ydata.
                        x_data.append(np.float(working_params[3]))
                        y_data.append(aniso)
                    else:
                        print "anisotropy called at line# ", line_counter, "missing blank, G-blank or G-factor numbers."
# If it is blank, g-blank or g-factor data, save it in the appropriate place. 
                elif working_params[3] in ["blank", "Blank", "BLANK"]:
                    blank_inten = [dumx, dumy]
                elif working_params[3] in ["g_blank", "G_blank", "G-blank", "g-blank", "G-Blank", "G_Blank"]:
                    g_blank_inten = [dumx, dumy]
                elif working_params[3] in ["g_factor", "G_Factor", "g-factor", "G-factor", "G-FACTOR", "G_FACTOR"]:
                    g_inten = [dumx, dumy]
# otherwise, this is probably caused by a typo in the setupfile, complain and say where.
                else:
                    print "Unknown value in anisotropy call.  Line# ",line_counter
                    continue

# 'intensity_timecourse' specific information.  
# The most relevant point about this being a timecourse is that there is the posibility of specifying a deadtime.
# the deadtime is given as working_params[3], the fourth parameter.  It must be a number for obvious reasons.  
# If the deadtime is not given, or given incorrectly, zero will be used.
# PTI_inten_text_raw is data copied from Felix and pasted into a text file.  there are only two columns, this is
# 'uncorrected data'
# PTI_inten_text_corr is the same, but with the intensity correct applied, and used here.
# PIT_inten_text_corr_but_use_raw is the same, but with the intensity correction appied, and ignored here.
# Otherwise, we're in generic two column mode.
            if data_type == "intensity_timecourse":
                deadtime = 0
                if data_file_type == "PTI_inten_text_raw":
                    time_data, inten_data = np.loadtxt(data_file, skiprows=4, unpack=True)
                elif data_file_type == "PTI_inten_text_corr":
                    dumtime, duminten, dumtime2, duminten2, time_data, inten_data = np.loadtxt(data_file, skiprows=4, unpack=True)
                elif data_file_type == "PTI_inten_text_corr_but_use_raw":
                    time_data, inten_data, dumtime, duminten, dumtime2, duminten2 = np.loadtxt(data_file, skiprows=4, unpack=True)
                else:
                    time_data, inten_data = np.loadtxt(data_file, unpack=True)
# check to see if the deadtime was given.
                try:
                    deadtime = np.float(working_params[3])
# if it is missing outright, complain but use zero.
                except IndexError:
                    print "No deadtime give for ", datasetname, " using 0.  Line# ",line_counter
# if it isn't a number, complain, and use zero
                except ValueError:
                    print "Deadtime for ",datasetname, " is not a number.  Line# ",line_counter
# apply the deadtime correction
                x_data = np.fromiter(time_data, float)
                x_data = x_data + deadtime*np.ones_like(x_data)
# assign the intensity data to y-data
                y_data = inten_data
# the 'marked_up_data' data type:  A complex data type that may become more common as I go forward.  Basically, it is 
# not formatted such that specific things need to be in specific places, but instead formatted such that specific things
# have specific keys associated with them.  This is not especially diskspace or processing time conservative, but makes 
# for somewhat more readable data sets. Also I may add XML and relatives here at some point.
            if data_type == "marked_up_data":
                if data_file_type == "KeyValuePairs":     
                    try:
                        xkey = working_params[3]
                        ykey = working_params[4]
                    except IndexError:
                        print "at line# ",line_counter,"Key name for x-values or y-values were expected, but not given."
                    dumnum = 0
                    dictdata = []
                    for line in open(data_file,'r').readlines():
                        kvpline = line.split()
                        dumdict = dict()
                        for elem in kvpline:
                            temp = elem.split('=')
                            try:
                                dumdict[temp[0]] = float(temp[1])
                            except ValueError:
                                dumdict[temp[0]] = temp[1]
                            except IndexError:
                                print "At line# ",dumnum+1," in ",data_file," please pass key-value pairs using the format key=value. No white space allowed in the key=pairs."
                                continue
                        dictdata.append(dumdict)
                        dumnum = dumnum + 1
                    for num in range(len(dictdata)):
                        dumdict = dictdata[num]
                        x_data.append(dumdict[xkey])
                        y_data.append(dumdict[ykey])

                elif data_file_type == "XML":
                    print "XML not supported yet, sorry."
                    pass
                else:
                    print "At line# ",line_counter," unrecognized data file type"
# and a sort of catchall data type.  Processed 2-column data can be loaded in so we can see their fit and plot them.
            if data_type == "processed_data_generic_2col":
                x_data, y_data = np.loadtxt(data_file, unpack=True)
# now close the if-else series that began by testing to see if the data set was open.                    
        else:
            print "At line# ", line_counter, " tried to add data without an open dataset."

# The print_dataset mode.  Simply put, this prints the current data set to the standard output.
# At the moment this whole program can only deal with x,y data.  As such, print_dataset assume the current 
# data set follows the convention of independent variable in x_data, and dependent in y_data.
    elif working_mode == "print_dataset":
        print "Dataset name:  ", datasetname
        dum = np.column_stack((x_data, y_data))
        for line in dum:
            print line[0], "     ",line[1]

# The save_dataset mode.  
# Yet to be implemented.  
# I'd like save_dataset and a filename, and it write the 
# dataset out to that.  I'd also like for it to check if it can write whereever it has been told, and make sure
# that there isn't a file of that name yet.  
# Perhaps a clobber okay flag as a final parameter to pass. The way I've used the program so far, I frequently 
# need to rerun a script, and if I need to delete a file, or rename a file everytime, I'll drive myself nuts.
# Perhaps a third mode, which adds a number to the end of the filename and increments it until it is new.

# The fit_dataset mode.  This takes the current dataset and fits it to a model specified by the first parameter
# At this point arbitrary equations aren't allowed.  Instead one of a series of prespecified models is used.  
# There are three results from the fit.  First is the best fit value and the +/- 1 SEM confidence interval (from the 
# variance in the covariance matrix.  This is printed directly to the stdout in a human readable format.
# Second is the optimized parameters are stored in popt for later use.  Third the fit_model is also stored for
# later use.  These latter two are used to generate the y_fit for plot_fit.
#
# An arbitrary number of models can be specified, they are checked against a list of possible models.  Each has
# a different number of parameters, which have meaning spcific to the specific model.
    elif working_mode == "fit_dataset":
# The direct_binding model:  Direct_binding is the binding of a receptor to a reporter ligand.  The actual output is 
# unimportant, although we typically use fluorescencne anisotropy for this.  As such a fluorescence anisotropy specific 
# casting of the observable equation is used below.  If straight intensity is used, this will likely need a recasting.
# Note to self, recast this to have different observable modes, we shouldn't hard code in the anisotropy style observable.
# direct_binding takes four parameters:  First is the concentration of the reporter in units consistent with the independent
# variable.  Then there are initial guesses for the Kd-reporter-receptor value, in the same units as the 
# independent variable, initial guesses for the observable value in the free state at the bound state.
# A final point, I'm actually using the compbindeq() function here, which is the solution for the competition binding 
# equilibrium, but I've set ligand concentration to zero.  K_lig_dummy isn't used here.
        if working_params[0] == "direct_binding":
            K_lig_dummy = 0.5
            reporter_conc_tot = float(working_params[1])
            initial_guess = [float(working_params[2]),float(working_params[3]),float(working_params[4])]
            def fit_model(recpt_tot, K_ligstar, obs_free, obs_bound):
                return (obs_free*compbindeq(reporter_conc_tot, 0., recpt_tot, K_ligstar, K_lig_dummy)[0]+obs_bound*compbindeq(reporter_conc_tot, 0., recpt_tot, K_ligstar, K_lig_dummy)[3]) / reporter_conc_tot
            popt, pcov = curve_fit(fit_model,x_data,y_data, initial_guess)
            have_fit = 1
            err_est = np.sqrt(np.diagonal(pcov))
            print "Results from fitting ",datasetname," with the direct_binding method:"
            print "Results are best fit for each parameter, +/- 1 sigma error."
            print "reporter-receptor affinity (Kd): ",popt[0], " +/- ",err_est[0]
            print "observable in free state: ", popt[1], " +/- ", err_est[1]
            print "observable in bound state: ", popt[2], " +/- ", err_est[2]
            print "Parameter covariance matrix is below"
            print pcov
# direct_binding_fixed_obs is the same as direct binding, but the two 'initial guess' for the observable values become
# the hardcoded values for the observable.  This would be used in cases where you have good estimates of these values 
# and thus don't want to fit them.  Same basic model, same equation used, same caveats about the observable, but the 
# observable values are not fit by curve_fit, instead they are hard_coded.
        elif working_params[0] == "direct_binding_fixed_obs":
            K_lig_dummy = 0.5
            reporter_conc_tot = float(working_params[1])
            obs_free = float(working_params[3])
            obs_bound = float(working_params[4])
            initial_guess = [float(working_params[2])]
            def fit_model(recpt_tot, K_ligstar):
                return (obs_free*compbindeq(reporter_conc_tot, 0., recpt_tot, K_ligstar, K_lig_dummy)[0]+obs_bound*compbindeq(reporter_conc_tot, 0., recpt_tot, K_ligstar, K_lig_dummy)[3]) / reporter_conc_tot
            popt, pcov = curve_fit(fit_model,x_data,y_data, initial_guess)
            have_fit = 1
            err_est = np.sqrt(np.diagonal(pcov))
            print "Results from fitting ",datasetname," with the direct_binding_fixed_obs method:"
            print "Results are best fit for each parameter, +/- 1 sigma error."
            print "reporter-receptor affinity (Kd): ",popt[0], " +/- ",err_est[0]
            print "Parameter covariance matrix is below"
            print pcov

# direct_binding_fixed_free is similar to direct_binding_fixed_obs.  Same number of parameters are passed, but the 
# obs_free is fixed, and obs_bound can float.  Generally, the free observable will be the easiest to get a good
# estimate for, and thus frequently one wants to fit the bound value, but not the free one.  Hence this model.  
# Same parameters and meaning as the other direct bind models.
        elif working_params[0] == "direct_binding_fixed_free":
            K_lig_dummy = 0.5
            reporter_conc_tot = float(working_params[1])
            obs_free = float(working_params[3])
            initial_guess = [float(working_params[2]),float(working_params[4])]
            def fit_model(recpt_tot, K_ligstar,obs_bound):
                return (obs_free*compbindeq(reporter_conc_tot, 0., recpt_tot, K_ligstar, K_lig_dummy)[0]+obs_bound*compbindeq(reporter_conc_tot, 0., recpt_tot, K_ligstar, K_lig_dummy)[3]) / reporter_conc_tot
            popt, pcov = curve_fit(fit_model,x_data,y_data, initial_guess)
            have_fit = 1
            err_est = np.sqrt(np.diagonal(pcov))
            print "Results from fitting ",datasetname," with the direct_binding_fixed_free method:"
            print "Results are best fit for each parameter, +/- 1 sigma error."
            print "reporter-receptor affinity (Kd): ",popt[0], " +/- ",err_est[0]
            print "observable in bound state: ", popt[1], " +/- ", err_est[1]
            print "Parameter covariance matrix is below"
            print pcov

# competition_binding model.  This is the most general, it will fit the obs_free and obs_bound, as well as the K_lig,
# where the latter is the competitive ligand Kd.  Important to realize here is that there is an assumption that only 
# ligand or reporter can bind at a time, and that there is only one binding site.
# Of note here, the receptor concentration needs to be fixed for the way this is set up.  In principle that
# isn't necessary, but it is hard coded into this particular model. 
# parameters passed are:  competition_binding  total_reporter_conc  total_receptor_conc  reporter-receptor_Kd  
# then guesses for Kd_comp_ligand  obs_free and obs_bound 
        elif working_params[0] == "competition_binding":
            reporter_conc_tot = float(working_params[1])
            receptor_conc_tot = float(working_params[2])
            K_ligstar = float(working_params[3])
            initial_guess = [float(working_params[4]),float(working_params[5]),float(working_params[6])]
            def fit_model(lig_tot, K_lig, obs_free, obs_bound):
                return (obs_free*compbindeq(reporter_conc_tot, lig_tot, receptor_conc_tot, K_ligstar, K_lig)[0]+obs_bound*compbindeq(reporter_conc_tot, lig_tot, receptor_conc_tot, K_ligstar, K_lig)[3]) / reporter_conc_tot
            popt, pcov = curve_fit(fit_model,x_data,y_data, initial_guess)
            have_fit = 1
            err_est = np.sqrt(np.diagonal(pcov))
            print "Results from fitting ",datasetname," with the competition_binding method:"
            print "Results are best fit for each parameter, +/- 1 sigma error."
            print "ligand-receptor affinity (Kd): ",popt[0], " +/- ",err_est[0]
            print "observable in free state: ", popt[1], " +/- ", err_est[1]
            print "observable in bound state: ", popt[2], " +/- ", err_est[2]
            print "Parameter covariance matrix is below"
            print pcov

# competition_binding_fixed_obs.  Same as competition_binding, but the obs_free and obs_bound are fixed at the values given.
        elif working_params[0] == "competition_binding_fixed_obs":
            reporter_conc_tot = float(working_params[1])
            receptor_conc_tot = float(working_params[2])
            K_ligstar = float(working_params[3])
            obs_free = float(working_params[5])
            obs_bound = float(working_params[6])
            initial_guess = [float(working_params[4])]
            def fit_model(lig_tot, K_lig):
                return (obs_free*compbindeq(reporter_conc_tot, lig_tot, receptor_conc_tot, K_ligstar, K_lig)[0]+obs_bound*compbindeq(reporter_conc_tot, lig_tot, receptor_conc_tot, K_ligstar, K_lig)[3]) / reporter_conc_tot
            popt, pcov = curve_fit(fit_model,x_data,y_data, initial_guess)
            have_fit = 1
            err_est = np.sqrt(np.diagonal(pcov))
            print "Results from fitting ",datasetname," with the competition_binding_fixed_obs method:"
            print "Results are best fit for each parameter, +/- 1 sigma error."
            print "ligand-receptor affinity (Kd): ",popt[0], " +/- ",err_est[0]
            print "Parameter covariance matrix is below"
            print pcov

# the catchall if the fit_model is unrcognized.
        else:
            print "Unknown fit_dataset mode used at line# ",line_counter

# this is a quick flag that will allow showing only a portion of the data
    elif working_mode == "set_plot_limits":
# mode one is when one argument is passed, this will set the maximum value for x axis 
        if len(working_params) == 1:
            plt.xlim(xmax=np.float(working_params[0]))
# mode two is when two arguments are passed, this will set x-axis min and max
        elif len(working_params) == 2:
            plt.xlim(xmin=np.float(working_params[0]))
            plt.xlim(xmax=np.float(working_params[1]))
# mode four is when four arguments are passed, this will set the x-min, x-max, y-min and y-max limits
        elif len(working_params) == 4:
            plt.xlim(xmin=np.float(working_params[0]))
            plt.xlim(xmax=np.float(working_params[1]))
            plt.ylim(ymin=np.float(working_params[2]))
            plt.ylim(ymax=np.float(working_params[3]))
        else:
            print "At line# ",line_counter," unrecognized number of arguments passed to set_data_limits."

# The plot_data command.  This started out very simple.  plot_data is called with a symbol for the data.
# I've since extended it slightly.  There is now a default, simple mode (call it with the symbol to use) and
# a 'verbose' mode.  If instead of a symbol, you give it a word like explicit, detailed or verbose, you can then
# pass plt.plot as many keywords as you want.  This allows more direct access to the flexibility of plot, 
# but only when you need it. This has thus far been of most use when you want to have a line, and markers,
# but only want a marker on every Nth data point.  The other use has been for changing marker sizes.
    elif working_mode == "plot_data":
        try:
            if working_params[0] in ('Explicit','explicit','Detailed','detailed','Verbose','verbose'):
                opts = dict()
                opts['plotmode']='linear'
                opts['label'] = datasetname
                opt_mode = working_params.pop(0)
                for param in working_params:
                    working = param.split('=')
                    try:
                        opts[working[0]] = float(working[1])
                    except ValueError:
                        opts[working[0]] = working[1]
                    except IndexError:
                        print "At line# ",line_counter," Please pass plot_data ",opt_mode," using opt=val"
                        continue
                plotmode=opts.pop('plotmode')
                if plotmode in ('linear','Linear','normal','nonlog'):
                    plt.plot(x_data,y_data,**opts)
                elif plotmode in ('semilog_x','semilogx','Semilogx','SemiLogX','Semilog_x','SemiLog_x','SemiLog_X'):
                    if np.alltrue(np.greater(x_data,0)):
                        plt.semilogx(x_data,y_data,**opts)
                    else: 
                        print "plotmode semilogx called with x-data not all being positive at line# ",line_counter
                elif plotmode in ('semilog_y','semilogy','Semilogy','SemiLogY','Semilog_y','SemiLog_y','SemiLog_Y'):
                    if np.alltrue(np.greater(y_data,0)):
                        plt.semilogy(x_data,y_data,**opts)
                    else: 
                        print "plotmode semilogy called with y-data not all being positive at line# ",line_counter
                elif plotmode in ('loglog','log-log','LogLog','Log-Log','logarithmic_scale','logarithmic'):
                    if np.alltrue(np.greater(y_data,0)) and np.alltrue(np.greater(x_data,0)):
                        plt.loglog(x_data,y_data,**opts)
                    else: 
                        print "plotmode loglog called with x- and y-data not all being positive at line# ",line_counter
                else:
                    print "Unrecognized plotmode at line # ", line_counter
            else:
                symbol = working_params[0]
                plt.plot(x_data,y_data,symbol,label=datasetname)
        except IndexError:
            print "No parameters given to plot_data at line# ",line_counter
            continue

# The plot_fit command:  This does more or less what it says it does.  Basically, it takes the x_data, and finds a range 
# to calculate y_values for, using the x-values, and the optimized parameters (in popt).  These are passed to the 
# model (fit_model) and y_values are calculated at enough points to get a nice smooth curve.  The command can be called
# in one of three models.  If one parameter is passed, it is the plot symbol (usually some color of line).  If three
# parameters are passed, they are the plot symbol, followed by the minimum and maximum x-values to plot over.  These 
# override the behavior that find the x-range from the min and max of the x-data.  Finally, if four parameters are passed
# the last one is the number of samplings.  If there is a particularly interesting behavior in the curve, it may
# require more samplings to get a smooth curve, hence this ability.
    elif working_mode == "plot_fit_old":
# note also that if a fit has not yet been performed it will complain but not fail.
        if have_fit == 0:
            print "at line# ",line_counter, " plot_fit called without current fit availible."
            continue
# set default values
        xmin = min(x_data)
        xmax = max(x_data)
        samples = 200
        symbol = 'k-'
# figure out how many parameters were passed, override default values accordingly
        if len(working_params) == 1:
            symbol = working_params[0]
        if len(working_params) == 3:
            symbol = working_params[0]
            xmin = float(working_params[1])
            xmax = float(working_params[2])
        if len(working_params) == 4:
            symbol = working_params[0]
            xmin = float(working_params[1])
            xmax = float(working_params[2])
            samples = float(working_params[3])
# create the x-values to use, initialize the y-values array.
        x_fit = np.linspace(xmin,xmax,samples)
        y_fit = []
# Below is a REALLY dumb way to pass the right number of arguments to fit_model.  Now I realize I 
# could have done this in two lines using *popt.  Leaving it for now. 
        if len(popt) == 1:
            for num in x_fit:
                y_fit.append(fit_model(num, popt[0]))
        elif len(popt) == 2:
            for num in x_fit:
                y_fit.append(fit_model(num, popt[0], popt[1]))
        elif len(popt) == 3:
            for num in x_fit:
                y_fit.append(fit_model(num, popt[0], popt[1], popt[2]))
        elif len(popt) == 4:
            for num in x_fit:
                y_fit.append(fit_model(num, popt[0], popt[1], popt[2], popt[3]))
        elif len(popt) == 5:
            for num in x_fit:
                y_fit.append(fit_model(num, popt[0], popt[1], popt[2], popt[3], popt[4]))
        else:
            print "plot_fit called with more than five parameters optimized, not supported.  Line# ",line_counter
# it is possible to run the above type of operation and not have a proper np.array at the end.  
# This simply forces it to a floating point np.array.  Thus plt.plot will work.  The last parameter in 
# the plt.plot call removes the fit from the legend.  I may want to change this later to an option.
        y_fit = np.fromiter(y_fit, float)
        plt.plot(x_fit,y_fit,symbol,label='_nolegend_')

# The plot_fit command:  This has been updated with a few new behaviors.
# Overall borrows a lot from plot_fit, but adds semilog and loglog plots.  Also cleans up the
# code a little.  I've made this version of plot_fit should maintain back compatibility, and escape out with
# a 'verbose' flag. 
#
# The behavior borrowed from the older flavor of plot_fit:
#
# This does more or less what it says it does.  Basically, it takes the x_data, and finds a range 
# to calculate y_values for, using the x-values, and the optimized parameters (in popt).  These are passed to the 
# model (fit_model) and y_values are calculated at enough points to get a nice smooth curve.  The command can be called
# in one of three models.  If one parameter is passed, it is the plot symbol (usually some color of line).  If three
# parameters are passed, they are the plot symbol, followed by the minimum and maximum x-values to plot over.  These 
# override the behavior that find the x-range from the min and max of the x-data.  Finally, if four parameters are passed
# the last one is the number of samplings.  If there is a particularly interesting behavior in the curve, it may
# require more samplings to get a smooth curve, hence this ability.
#
# Some added behavior.  
# I've added the ability to escape from the old behavior by calling plot_fit with a verbose flag.
# When this is done, it exposes the new behaviors and plotmode options.
# if the first option is 'verbose' or related, then everything must be specified manually.  This improves
# readability anyhow, so it is worth adding.  If in the 'verbose' mode, then semilog etc. can be 
# specified for the plotmode.
    elif working_mode == "plot_fit":
# note also that if a fit has not yet been performed it will complain but not fail.
        if have_fit == 0:
            print "at line# ",line_counter, " plot_fit called without current fit availible."
            continue
# set default values
        if working_params[0] in ('Explicit','explicit','Detailed','detailed','Verbose','verbose'):
            working_params.pop(0)  # get rid of the verbose flag 
            fitopts=dict()  # initialize the option dictionary, then populate it with defaults
            fitopts['xmin']=min(x_data)
            fitopts['xmax']=max(x_data)
            fitopts['samples']=200
            fitopts['symbol']='k-'
            fitopts['plotmode']='linear'
            fitopts['sampling_mode']='linear'
            for num in range(len(working_params)):   # Get the options and pass them into the options dict.
                temp = working_params[num].split('=')
                try:
                    fitopts[temp[0]] = float(temp[1])
                except ValueError:
                    fitopts[temp[0]] = temp[1]
                except IndexError:
                    print "At line# ",line_counter," Please pass plot_fit options using the format: option=value"
                    continue
### probably best to pop the flags out of the options so I can then pass the remaining options to the plotting functions.
            sampling_mode = fitopts.pop('sampling_mode')
            plotmode = fitopts.pop('plotmode')
            symbol = fitopts.pop('symbol')
            samples = fitopts.pop('samples')
            xmin = fitopts.pop('xmin')
            xmax = fitopts.pop('xmax')
### Now decide how to sample the fit, and get the data points.
            if sampling_mode in ('linear','Linear','normal','nonlog','linear_x','lin_x','LinX','LINX'):
                x_fit = np.linspace(xmin, xmax, samples)
                y_fit = []
                for num in x_fit:
                    y_fit.append(fit_model(num, *popt))
            elif sampling_mode in ('semilog_x','semilogx','Semilogx','SemiLogX','Semilog_x','SemiLog_x','SemiLog_X','logx','Log_x','Log_X','LOGX','LogX','log_x'):
                if np.alltrue(np.greater(np.asarray([xmin, xmax]),0)):
                    x_fit = np.exp( np.linspace(np.log(xmin),np.log(xmax),samples) )
                    y_fit = []
                    for num in x_fit:
                        y_fit.append(fit_model(num, *popt))
                else:
                    print "logrithimic sampling of x-axis used with negative x-values at line# ",line_counter 
            else:
                print 'unrecognized sampling_mode at line# ',line_counter
### We have the data points, now determine the mode for plotting, do a sanity check, and plot the data.
            if plotmode in ('linear','Linear','normal','nonlog'):
                plt.plot(x_fit,y_fit,**fitopts)
            elif plotmode in ('semilog_x','semilogx','Semilogx','SemiLogX','Semilog_x','SemiLog_x','SemiLog_X'):
                if np.alltrue(np.greater(x_fit,0)):
                    plt.semilogx(x_fit,y_fit,**fitopts)
                else: 
                    print "plotmode semilogx called with x-data not all being positive at line# ",line_counter
            elif plotmode in ('semilog_y','semilogy','Semilogy','SemiLogY','Semilog_y','SemiLog_y','SemiLog_Y'):
                if np.alltrue(np.greater(y_fit,0)):
                    plt.semilogy(x_fit,y_fit,**fitopts)
                else: 
                    print "plotmode semilogy called with y-data not all being positive at line# ",line_counter
            elif plotmode in ('loglog','log-log','LogLog','Log-Log','logarithmic_scale','logarithmic'):
                if np.alltrue(np.greater(y_fit,0)) and np.alltrue(np.greater(x_fit,0)):
                    plt.loglog(x_fit,y_fit,**fitopts)
                else: 
                    print "plotmode loglog called with x- and y-data not all being positive at line# ",line_counter
            else:
                print "Unrecognized plotmode at line # ", line_counter
### Fall back to the old behavior if there is no 'verbose' flag
        else:
            xmin = min(x_data)
            xmax = max(x_data)
            samples = 200
            symbol = 'k-'
# figure out how many parameters were passed, override default values accordingly
            if len(working_params) == 0:
                pass
            elif len(working_params) == 1:
                symbol = working_params[0]
            elif len(working_params) == 3:
                symbol = working_params[0]
                xmin = float(working_params[1])
                xmax = float(working_params[2])
            elif len(working_params) == 4:
                symbol = working_params[0]
                xmin = float(working_params[1])
                xmax = float(working_params[2])
                samples = float(working_params[3])
            else:
                print "at line # ",line_counter," plot_fit called with unrecognized parameter format."
# create the x-values to use, initialize the y-values array.
            x_fit = np.linspace(xmin,xmax,samples)
            y_fit = []
            for num in x_fit:
                y_fit.append(fit_model(num, *popt))
# it is possible to run the above type of operation and not have a proper np.array at the end.  
# This simply forces it to a floating point np.array.  Thus plt.plot will work.  The last parameter in 
# the plt.plot call removes the fit from the legend.  I may want to change this later to an option.
            y_fit = np.fromiter(y_fit, float)
            plt.plot(x_fit,y_fit,symbol,label='_nolegend_')

# print_fit_points prints out data points for the current fit.  Given that some of the fitting is easy here
# but hard elsewhere, this will allow folks to save the data and plot it as desired elsewhere.  
    elif working_mode == "print_fit_points":
        print "Datapoints for a curve fit to dataset name:  ", datasetname
        if have_fit == 0:
            print "at line# ",line_counter, " print_fit_points called without current fit availible."
            continue
# set default values
        if working_params[0] in ('Explicit','explicit','Detailed','detailed','Verbose','verbose'):
            working_params.pop(0)  # get rid of the verbose flag 
            fitopts=dict()  # initialize the option dictionary, then populate it with defaults
            fitopts['xmin']=min(x_data)
            fitopts['xmax']=max(x_data)
            fitopts['samples']=200
            fitopts['symbol']='k-'
            fitopts['sampling_mode']='linear'
            for num in range(len(working_params)):   # Get the options and pass them into the options dict.
                temp = working_params[num].split('=')
                try:
                    fitopts[temp[0]] = float(temp[1])
                except ValueError:
                    fitopts[temp[0]] = temp[1]
                except IndexError:
                    print "At line# ",line_counter," Please pass plot_fit options using the format: option=value"
                    continue
### probably best to pop the flags out of the options so I can then pass the remaining options to the plotting functions.
            sampling_mode = fitopts.pop('sampling_mode')
            symbol = fitopts.pop('symbol')
            samples = fitopts.pop('samples')
            xmin = fitopts.pop('xmin')
            xmax = fitopts.pop('xmax')
### Now decide how to sample the fit, and get the data points.
            if sampling_mode in ('linear','Linear','normal','nonlog','linear_x','lin_x','LinX','LINX'):
                x_fit = np.linspace(xmin, xmax, samples)
                y_fit = []
                for num in x_fit:
                    y_fit.append(fit_model(num, *popt))
            elif sampling_mode in ('semilog_x','semilogx','Semilogx','SemiLogX','Semilog_x','SemiLog_x','SemiLog_X','logx','Log_x','Log_X','LOGX','LogX','log_x'):
                if np.alltrue(np.greater(np.asarray([xmin, xmax]),0)):
                    x_fit = np.exp( np.linspace(np.log(xmin),np.log(xmax),samples) )
                    y_fit = []
                    for num in x_fit:
                        y_fit.append(fit_model(num, *popt))
                else:
                    print "logrithimic sampling of x-axis used with negative x-values at line# ",line_counter 
            else:
                print 'unrecognized sampling_mode at line# ',line_counter
### Fall back to the old behavior if there is no 'verbose' flag
        else:
            xmin = min(x_data)
            xmax = max(x_data)
            samples = 200
            symbol = 'k-'
# figure out how many parameters were passed, override default values accordingly
            if len(working_params) == 0:
                pass
            elif len(working_params) == 1:
                symbol = working_params[0]
            elif len(working_params) == 3:
                symbol = working_params[0]
                xmin = float(working_params[1])
                xmax = float(working_params[2])
            elif len(working_params) == 4:
                symbol = working_params[0]
                xmin = float(working_params[1])
                xmax = float(working_params[2])
                samples = float(working_params[3])
            else:
                print "at line # ",line_counter," plot_fit called with unrecognized parameter format."
# create the x-values to use, initialize the y-values array.
            x_fit = np.linspace(xmin,xmax,samples)
            y_fit = []
            for num in x_fit:
                y_fit.append(fit_model(num, *popt))

# it is possible to run the above type of operation and not have a proper np.array at the end.  
# This simply forces it to a floating point np.array.  Thus plt.plot will work.  The last parameter in 
# the plt.plot call removes the fit from the legend.  I may want to change this later to an option.
        y_fit = np.fromiter(y_fit, float)
        dum = np.column_stack((x_fit, y_fit))
        for line in dum:
            print line[0], "     ",line[1]
# okay, here's one of those things matplotlib is good at.  After all the worrying, adding a legend it trivial.
# loc=0 has the legend artist pick the best location based on were the curves end up.  Note that if you save it as
# an .eps you can move the legend, and the curves will still be present if they were initially obscured.  Seems like
# the most straightforward approach.  So far, when loc=0 has put the legend on top of a curve, I haven't had a better
# location in the frame.  Placing the legend outside of the frame is possible but a hassle.
    elif working_mode == "add_legend":
        plt.legend(loc=0)

# The show_figure command.  Again, made very easy by matplotlib, and the way everything else has been set up.
# if you recall xaxislabel and yaxislabel are drawn from the options when the data_set was opened initially.
# show_figure otherwise just shows the figure.  If the viewable range has been altered anywhere in the setupfile, 
# it will persist through out.  I'm actually not sure that that is supposed to happen, but it makes sense for it
# to do so in my head.
    elif working_mode == "show_figure":
        plt.xlabel(xaxislabel)
        plt.ylabel(yaxislabel)

        plt.show()

# The save_figure command.  Also made very easy by matplotlib and the above structure.
# more or less the same rules apply here that applied to show_figure.  There is one extension.  The figure
# is saved as it would have been  shown.  The file name is the one parameter passed to it, and the .xxx extention
# dictates the file type.  Examples: .eps wil same it as an 'eps' file (ie. as vector art) and .png saves it as a 'png'
    elif working_mode == "save_figure":  
        plt.xlabel(xaxislabel)
        plt.ylabel(yaxislabel)
        plt.savefig(working_params[0])

# The clear_figure command.  This is just a command to clear the figure.  Matplotlib builds up a figure from commands
# and is only sort of capable of dealing with multiple figures in the mode I'm using it.  Thus we can make and save
# a series of figures, but we'll want to clear out the one we're done with after we've saved it.  This command
# achieves that.
    elif working_mode == "clear_figure":
        plt.clf()

# The meta_data command.  This function writes 'meta data' as key value pairs to a dictionary that can be unpacked and 
# written to the command line under various circumstances.  The first place to do this will be with the AAAanalyze
# verbose key output_format.

    elif working_mode == "meta_data":
#        print len(working_params)
#        print range(len(working_params))
        for num in range(len(working_params)):
            temp = working_params[num].split('=')
            try:
                metadata[temp[0]] = float(temp[1])
            except ValueError:
                metadata[temp[0]] = temp[1]
            except IndexError:
                print "At line# ",line_counter," Please pass AAAanalyze ",opt_mode," options using the format: option=value"
                continue

# the print_meta_data mode.  This just allows one to output the 'meta_data' information in a specified format.  
    elif working_mode == 'print_meta_data':
        try:
            if working_params[0] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                metadatastring = ''
                for key in metadata:
                    metadatastring = metadatastring+key+'='+repr(metadata[key])+'   '
                print metadatastring
            else: 
                print 'unrecognized meta data output format:  %(outf)s  given at line# %(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
        except IndexError:
            print "At Line # ", line_counter, " no format specified for print_meta_data, meta_data is: ", metadata 

        

# The AAAanalyze command.  This function has migrated over from the older AAAanalyze package.   A substantial rewrite 
# of the code has occurred in the process.  This is partially to capitalize on/adapt to the functions in numpy and scipy. 
# Also I'll add the ability to graphically show what the functions are doing, and I'm going to try for a more flexible
# 'override' system, instead of a separate function for each type of processing.

# A bit about the layout of this command.  First it has a few subcommands, which correspond to the various modes in the 
# original AAAanalyze script.  Second, one can get out of the basic forms of those modes using a subcommand of verbose etc.
# The verbose escape command comes before the subcommand and parameters for the subcommand.
# in the 'verbose' mode, you can pass override commands, eg. "max_inten=10" as a series of key=value pairs, after the 
# required parameters for the subcommand.  An example:
# AAAanalyze  verbose  BE_t50  10.0  1.0  4.0   5  0.03   max_inten=10

    elif working_mode == "AAAanalyze":
        try:                             # This try statement is to allow an error handling of calling AAAanalyze without any parameters
            x_data2 = np.copy(x_data)    # Initialize the data set, this allows masking later
            y_data2 = np.copy(y_data)
            overrides = dict()           # This is where the override commands will go, if used.

# This 'if block' handles the override commands.  Note that it removes the verbose etc. flag at the front, and the overrides are stripped off.  
# The block ends without and else/elif such that we can enter the if...elif...else AAAanalyze mode block like normal.  Then
# we can process the data as normal and apply the overrides at an appropriate point.

            if working_params[0] in ('Override','override','Explicit','explicit','Detailed','detailed','Verbose','verbose'):
                opt_mode = working_params.pop(0)
                while '=' in working_params[-1]:
                    temp = working_params.pop(-1).split('=')
                    try:
                        overrides[temp[0]] = float(temp[1])
                    except ValueError:
                        overrides[temp[0]] = temp[1]
                    except IndexError:
                        print "At line# ",line_counter," Please pass AAAanalyze ",opt_mode," options using the format: option=value"
                        continue

# This is a trouble shooting trick to get it to print out the overrides.
#                for key in overrides:
#                    print "Key:  ", key,"   Value:  ",overrides[key]

# The 'mask_data' override is handled by changing the data passed to the AAAanalyze functions.  This is why we are using x_data2 and y_data2 instead of x_data/y_data
# The mask_data=intervals format for intervals is rigid.  It is "beg"-"end" clauses stuck together with ","s.  Beg and end are numbers indicating ranges
# of x_values to mask out.  An interval is indicated as 'through' using the dash ('-').
                if 'mask_data' in overrides:
                    intervals = overrides.pop('mask_data').split(',')         # Splits up the statement into intervals, separating at ','
                    mask = np.isnan(np.ones_like(x_data))                     # This is just a way to initialize an array of the correct size with boolean False.
                    while len(intervals)>0:                                   # Allows an arbitrary number of intervals  
                        temp = intervals.pop(0).split('-')                    # Separating intervals into beg and end, by using the dash
                        tempmask = x_data.__gt__(float(temp[0])).__and__(x_data.__lt__(float(temp[1])))    # use __and__ to get intervals with boolean masks
                        mask = mask.__or__(tempmask)                                                       # use __or__ to get union of intervals
                    x_data2 = x_data[np.logical_not(mask)]                                                 # actually mask out the data we don't want to process
                    y_data2 = y_data[np.logical_not(mask)]
                    pass_to_visproc['data_masked_out'] = [x_data[mask],y_data[mask]]                       # and pass the masked out data to pass_to_visproc
                else:
                    pass
# something of a reformat here.  We'll check if the verbose etc. flag is given.  Then remove it at the same time as constructing an override dict()
# by removing overrides from the end of the command.  Then we start the if...elif...else loop from the remaining command which should now be formatted
# properly.  Then I need to add, within each block of the if...elif...else loop a means of checking and applying the overrides.   

            if working_params[0] in ('BarbedEnds','BE_t50','BarbedEnds_t50','Find_BE_t50','Find_BEs_t50','BEs_t50'):        
                opts = dict()
                opts['total_actin'] = float(working_params[1])
                opts['kon'] = float(working_params[2])
                opts['koff'] = float(working_params[3])
                opts['critical_conc'] = opts['koff'] / opts['kon']

# Here's where the override options min_inten and max_inten are handled
                if 'min_inten' in overrides:
                    opts['min_inten'] = overrides.pop('min_inten')
                    pass_to_visproc['override_min'] = opts['min_inten']
                elif 'min_window' in overrides:        # uses the first 'n' points to get an average for min_inten
                    temp = int(overrides.pop('min_window'))
                    opts['min_inten'] = np.average(y_data2[:temp])
                    pass_to_visproc['averaged_min'] = opts['min_inten']
                    pass_to_visproc['points_to_find_min'] = [x_data2[:temp],y_data2[:temp]]
                else:
                    opts['min_inten'] = y_data2.min()
                    pass_to_visproc['absolute_min'] = opts['min_inten']

                if 'max_inten' in overrides:
                    opts['max_inten'] = overrides.pop('max_inten')
                    pass_to_visproc['override_max'] = opts['max_inten']
                elif 'points_to_find_max' in overrides:
                    pts = int(overrides.pop('points_to_find_max'))
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=pts, points_to_average=pts)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                else:
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=5, points_to_average=5)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc

                opts['window2'] = float(working_params[4])
                opts['switcher'] = 1
                result, temp_to_proc = find_bes_at_t50(x_data2, y_data2, **opts)
                for key in temp_to_proc:
                    pass_to_visproc[key] = temp_to_proc[key]
                del temp_to_proc
                if 'output_format' in overrides:
                    if overrides['output_format'] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                        overrides.pop('output_format')
                        metadatastring = ''
                        for key in metadata:
                            try:
                                metadatastring = metadatastring+key+'='+metadata[key]+'   '
                            except TypeError:
                                metadatastring = metadatastring+key+'='+repr(metadata[key])+'    '
                        print 'Line=%(lc)d   Filename=%(fn)s  BE_t50=%(res).16f  '% {'lc': line_counter,'fn': data_file,'res':result} , metadatastring
                    else: 
                        print 'unrecognized output_format:  %(outf)s  given at line# $(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
                else:
                    print "Line # ", line_counter, "  Filename:  ", data_file, "  Barbed ends at t50 is:  ", result

            elif working_params[0] in ('BarbedEnds_maxAP','BE_maxAP','bes_apmax','BEs_at_AP_max','BEs_at_APmax','bes_at_ap_max'):
                opts = dict()
                opts['total_actin'] = float(working_params[1])
                opts['kon'] = float(working_params[2])
                opts['koff'] = float(working_params[3])
                opts['critical_conc'] = opts['koff'] / opts['kon']
# Here's where the override options min_inten and max_inten are handled
                if 'min_inten' in overrides:
                    opts['min_inten'] = overrides.pop('min_inten')
                    pass_to_visproc['override_min'] = opts['min_inten']
                elif 'min_window' in overrides:        # uses the first 'n' points to get an average for min_inten
                    temp = int(overrides.pop('min_window'))
                    opts['min_inten'] = np.average(y_data2[:temp])
                    pass_to_visproc['averaged_min'] = opts['min_inten']
                    pass_to_visproc['points_to_find_min'] = [x_data2[:temp],y_data2[:temp]]
                else:
                    opts['min_inten'] = y_data2.min()
                    pass_to_visproc['absolute_min'] = opts['min_inten']
                if 'max_inten' in overrides:
                    opts['max_inten'] = overrides.pop('max_inten')
                    pass_to_visproc['override_max'] = opts['max_inten']
                elif 'points_to_find_max' in overrides:
                    pts = int(overrides.pop('points_to_find_max'))
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=pts, points_to_average=pts)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                else:
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=5, points_to_average=5)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc

                opts['window'] = int(working_params[4])
                result, temp_to_proc = bes_at_max_actin_poly(x_data2,y_data2,**opts)
                for key in temp_to_proc:
                    pass_to_visproc[key] = temp_to_proc[key]
                del temp_to_proc
                if 'output_format' in overrides:
                    if overrides['output_format'] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                        overrides.pop('output_format')
                        metadatastring = ''
                        for key in metadata:
                            try:
                                metadatastring = metadatastring+key+'='+metadata[key]+'   '
                            except TypeError:
                                metadatastring = metadatastring+key+'='+repr(metadata[key])+'    '
                        print 'Line=%(lc)d   Filename=%(fn)s  BEatAPmax=%(res).16f  '% {'lc': line_counter,'fn': data_file,'res':result} , metadatastring
                    else: 
                        print 'unrecognized output_format:  %(outf)s  given at line# $(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
                else:
                    print "Line # ", line_counter, " Filename:  ", data_file, "  Barbed ends at max actin polymerization rate are:  ", result

            elif working_params[0] in ('MaxAP','Max_Actin_Poly_Rate','APmax','Find_max_AP','Find_APmax','Find_AP_max'):
                opts = dict()
                opts['total_actin'] = float(working_params[1])
                kon = float(working_params[2])
                koff = float(working_params[3])
                opts['critical_conc'] = koff / kon
# Here's where the override options min_inten and max_inten are handled
                if 'min_inten' in overrides:
                    opts['min_inten'] = overrides.pop('min_inten')
                    pass_to_visproc['override_min'] = opts['min_inten']
                elif 'min_window' in overrides:        # uses the first 'n' points to get an average for min_inten
                    temp = int(overrides.pop('min_window'))
                    opts['min_inten'] = np.average(y_data2[:temp])
                    pass_to_visproc['averaged_min'] = opts['min_inten']
                    pass_to_visproc['points_to_find_min'] = [x_data2[:temp],y_data2[:temp]]
                else:
                    opts['min_inten'] = y_data2.min()
                    pass_to_visproc['absolute_min'] = opts['min_inten']
                if 'max_inten' in overrides:
                    opts['max_inten'] = overrides.pop('max_inten')
                    pass_to_visproc['override_max'] = opts['max_inten']
                elif 'points_to_find_max' in overrides:
                    pts = int(overrides.pop('points_to_find_max'))
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=pts, points_to_average=pts)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                else:
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=5, points_to_average=5)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                opts['window'] = int(working_params[4])
                result, temp_to_proc = max_actin_poly_rate(x_data2,y_data2,**opts)
                for key in temp_to_proc:
                    pass_to_visproc[key] = temp_to_proc[key]
                del temp_to_proc
                if 'output_format' in overrides:
                    if overrides['output_format'] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                        overrides.pop('output_format')
                        metadatastring = ''
                        for key in metadata:
                            try:
                                metadatastring = metadatastring+key+'='+metadata[key]+'   '
                            except TypeError:
                                metadatastring = metadatastring+key+'='+repr(metadata[key])+'    '
                        print 'Line=%(lc)d   Filename=%(fn)s  APmax=%(apm).16f  '% {'lc': line_counter,'fn': data_file,'apm':result} , metadatastring
                    else: 
                        print 'unrecognized output_format:  %(outf)s  given at line# $(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
                else:
                    print "Line # ", line_counter, " Filename:  ", data_file, "  Max actin polymerization rate is:  ", result

            elif working_params[0] in ('Find_t50','t50','half_time','T50','t_half','thalf','t_1/2'):
                opts = dict()
                opts['window'] = float(working_params[1])

# Here's where the override options min_inten and max_inten are handled
                if 'min_inten' in overrides:
                    opts['min_inten'] = overrides.pop('min_inten')
                    pass_to_visproc['override_min'] = opts['min_inten']
                elif 'min_window' in overrides:        # uses the first 'n' points to get an average for min_inten
                    temp = int(overrides.pop('min_window'))
                    opts['min_inten'] = np.average(y_data2[:temp])
                    pass_to_visproc['averaged_min'] = opts['min_inten']
                    pass_to_visproc['points_to_find_min'] = [x_data2[:temp],y_data2[:temp]]
                else:
                    opts['min_inten'] = y_data2.min()
                    pass_to_visproc['absolute_min'] = opts['min_inten']

                if 'max_inten' in overrides:
                    opts['max_inten'] = overrides.pop('max_inten')
                    pass_to_visproc['override_max'] = opts['max_inten']
                elif 'points_to_find_max' in overrides:
                    pts = int(overrides.pop('points_to_find_max'))
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=pts, points_to_average=pts)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                else:
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=5, points_to_average=5)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc

                result, temp_to_proc = find_t50(x_data2,y_data2,**opts)
                for key in temp_to_proc:
                    pass_to_visproc[key] = temp_to_proc[key]
                del temp_to_proc
                if 'output_format' in overrides:
                    if overrides['output_format'] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                        overrides.pop('output_format')
                        metadatastring = ''
                        for key in metadata:
                            try:
                                metadatastring = metadatastring+key+'='+metadata[key]+'   '
                            except TypeError:
                                metadatastring = metadatastring+key+'='+repr(metadata[key])+'    '
                        print 'Line=%(lc)d   Filename=%(fn)s  t50=%(nc)f   '% {'lc': line_counter,'fn': data_file,'nc':result} , metadatastring
                    else: 
                        print 'unrecognized output_format:  %(outf)s  given at line# $(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
                else:
                    print "AAAanalyze Find_t50:  t50 is:  ", result

            elif working_params[0] in ('ActinPolymerizationRate','ActinPolymerizationRate_t50','AP_t50','Find_AP_t50'):
                opts = dict()
                opts['total_actin'] = float(working_params[1])
                opts['kon'] = float(working_params[2])
                opts['koff'] = float(working_params[3])
                opts['critical_conc'] = opts['koff'] / opts['kon']

# Here's where the override options min_inten and max_inten are handled
                if 'min_inten' in overrides:
                    opts['min_inten'] = overrides.pop('min_inten')
                    pass_to_visproc['override_min'] = opts['min_inten']
                elif 'min_window' in overrides:        # uses the first 'n' points to get an average for min_inten
                    temp = int(overrides.pop('min_window'))
                    opts['min_inten'] = np.average(y_data2[:temp])
                    pass_to_visproc['averaged_min'] = opts['min_inten']
                    pass_to_visproc['points_to_find_min'] = [x_data2[:temp],y_data2[:temp]]
                else:
                    opts['min_inten'] = y_data2.min()
                    pass_to_visproc['absolute_min'] = opts['min_inten']

                if 'max_inten' in overrides:
                    opts['max_inten'] = overrides.pop('max_inten')
                    pass_to_visproc['override_max'] = opts['max_inten']
                elif 'points_to_find_max' in overrides:
                    pts = int(overrides.pop('points_to_find_max'))
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=pts, points_to_average=pts)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                else:
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=5, points_to_average=5)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc

                opts['window2'] = float(working_params[4])
                opts['switcher'] = 2
                result, temp_to_proc = find_bes_at_t50(x_data2, y_data2, **opts)
                for key in temp_to_proc:
                    pass_to_visproc[key] = temp_to_proc[key]
                del temp_to_proc
                if 'output_format' in overrides:
                    if overrides['output_format'] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                        overrides.pop('output_format')
                        metadatastring = ''
                        for key in metadata:
                            try:
                                metadatastring = metadatastring+key+'='+metadata[key]+'   '
                            except TypeError:
                                metadatastring = metadatastring+key+'='+repr(metadata[key])+'    '
                        print 'Line=%(lc)d   Filename=%(fn)s  APrate=%(res).16f  '% {'lc': line_counter,'fn': data_file,'res':result} , metadatastring
                    else:
                        print 'unrecognized output_format:  %(outf)s  given at line# $(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
                else:
                    print "Line # ", line_counter, "  Filename:  ", data_file, "  Actin Polymerization Rate at t50 is:  ", result

# Adding a new function, one that fits a parabola to the beginning of the curve to find the 'nucleation rate'.

            elif working_params[0] in ('ActinNucleationRate','Find_NucRate','Find_NR','Find_ANR','NucleationRate','NucRate','nucleationrate','nucrate'):
                opts = dict()
                opts['total_actin'] = float(working_params[1])
                opts['kon'] = float(working_params[2])
                opts['koff'] = float(working_params[3])
                opts['critical_conc'] = opts['koff'] / opts['kon']

# Here's where the override options min_inten and max_inten are handled
                if 'min_inten' in overrides:
                    opts['min_inten'] = overrides.pop('min_inten')
                    pass_to_visproc['override_min'] = opts['min_inten']
                elif 'min_window' in overrides:        # uses the first 'n' points to get an average for min_inten
                    temp = int(overrides.pop('min_window'))
                    opts['min_inten'] = np.average(y_data2[:temp])
                    pass_to_visproc['averaged_min'] = opts['min_inten']
                    pass_to_visproc['points_to_find_min'] = [x_data2[:temp],y_data2[:temp]]
                else:
                    opts['min_inten'] = y_data2.min()
                    pass_to_visproc['absolute_min'] = opts['min_inten']

                if 'max_inten' in overrides:
                    opts['max_inten'] = overrides.pop('max_inten')
                    pass_to_visproc['override_max'] = opts['max_inten']
                elif 'points_to_find_max' in overrides:
                    pts = int(overrides.pop('points_to_find_max'))
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=pts, points_to_average=pts)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc
                else:
                    opts['max_inten'], temp_to_proc = find_max_inten(x_data2, y_data2, times_near_max=5, points_to_average=5)
                    for key in temp_to_proc:
                        pass_to_visproc[key] = temp_to_proc[key]
                    del temp_to_proc

                opts['inten_window'] = float(working_params[4])
                nuc_rate,bes_t0,fila_t0, temp_to_proc = find_actin_nuc_rate(x_data2, y_data2, **opts)
                for key in temp_to_proc:
                    pass_to_visproc[key] = temp_to_proc[key]
                del temp_to_proc
                if 'output_format' in overrides:
                    if overrides['output_format'] in ('kvpairs','key_value_pairs','kvp','kvps','KVP','KeyValuePairs'):
                        overrides.pop('output_format')
                        metadatastring = ''
                        for key in metadata:
                            try:
                                metadatastring = metadatastring+key+'='+metadata[key]+'   '
                            except TypeError:
                                metadatastring = metadatastring+key+'='+repr(metadata[key])+'    '
                        print 'Line=%(lc)d   Filename=%(fn)s  InitActinNuclRate=%(nc).16f   BEt0=%(be0).16f   filt0=%(fat0).16f    '% {'lc': line_counter,'fn': data_file,'nc':nuc_rate, 'be0':bes_t0, 'fat0':fila_t0} , metadatastring
                    else:
                        print 'unrecognized output_format:  %(outf)s  given at line# $(ln)d' % {'outf': overrides['output_format'],'ln':line_counter}
                else:
                    print "Line # ", line_counter, "  Filename:  ", data_file, "  Initial Actin Nucleation Rate is:  ", nuc_rate
                    print "Barbed ends at t=0 is: ",bes_t0,"  (this should be close to zero, 0.001 is 1nM when units are uM, which is a lot)"
                    print "Filamentous actin at t=0 is: ",fila_t0," this should be close to zero."


            else:
                pass
                print "Unrecognized AAAanalyze working mode ", working_params[0]," given at line# ",line_counter
            if len(overrides) > 0:
                print "Unrecognized override command(s) given:"
                for key in overrides:
                    print "Key: ", key, "  Value: ", overrides[key]
                overrides = dict()
                    
        except IndexError:
            print "No parameters given to AAAanalyze at line# ",line_counter
            continue


# The plot_slopes command.  One thing I've often found useful has been to plot the slopes of actin assembly assay data. 
# This function achieves that.  AAAanalyze calculates the slopes, thus this needs to be set up such that the slopes are 
# acquired from AAAanalyze, or to calculate them on the fly, but we'll need to pass 'plot_slopes' the window size to use
# when calculating slopes.

    elif working_mode == "plot_slopes":
        print "plot_slopes isn't implemented yet, sorry, SBP 3/9/2011"

# 'show_analysis' is to be the graphical display of how the analysis worked for a single dataset.  I'm attempting to make
# this general for a variety of analyses, instead of specific to each one.  The basic idea is that the analysis functions
# will build up dictionaries, where the 'key' is to be the label, and the value will either be xvals and yvals or a fixed
# yval.  
    elif working_mode == "show_analysis":
        plt.plot(x_data,y_data,'k-',label=datasetname)
        iter = 0
        plt.title(datasetname)
        colorlist = ['r','g','b','c','m','y','r','g','b','c','m','y','r','g','b','c','m','y','r','g','b','c','m','y','r','g','b','c','m','y','r','g','b','c','m','y']
        symbollist = ['o','s','*','v','^','D','<','>','+','h','p','o','s','*','v','^','D','<','>','+','h','p','o','s','*','v','^','D','<','>','+','h','p']
        for key in pass_to_visproc:
            if key in ['estimated_max','estimated_min','absolute_max','absolute_min','override_max','override_min','50%_intensity_change','min_intensity','averaged_min']:
                plt.hlines(pass_to_visproc[key],x_data.min(),x_data.max(),color=colorlist.pop(0),label=key)
            elif key in ['points_to_calc_max','points_to_calc_t50','points_to_calc_slope_t50','points_to_calc_slope_max','points_to_calc_max_slope','data_masked_out','points_to_find_min','points_to_calc_nuc_rate']:
                plt.plot(pass_to_visproc[key][0],pass_to_visproc[key][1],color=colorlist.pop(0),marker=symbollist.pop(0),linestyle='None',label=key)
            elif key in ['t50','t50b']:
                plt.vlines(pass_to_visproc[key],y_data.min(),y_data.max(),color=colorlist.pop(0),label=key)
            elif key in ['trend_at_max_slope','line_to_extrapolate_t50','parabola_for_nuc_rate']:
                dum = pass_to_visproc[key][0]
                tstart = pass_to_visproc[key][1]
                tend = pass_to_visproc[key][2]
                tvals = np.linspace(tstart, tend, 250)
                trendvals =  np.polyval(dum,tvals)
                plt.plot(tvals,trendvals,color=colorlist.pop(0),linestyle='--')
            else:
                print "unrecognized key passed to show_analysis, line#: ", line_counter
        pass_to_visproc = dict()
        
# The catchall complaint if there is a typo in the setupfile.  It wil complain and then continue.

# There is one danger here, a typo doesn't end execution.  I think I'm okay with that.  In my playing so far that has 
# yet to cause anything more than confusion.
    else:
        print "Unknown working mode at line# ", line_counter
        print working_command

# end of the script
